// give_bs_elt.hxx
//
// Mical, a regular grammar inference library
// Copyright (C) 2003 Maxime Rey
//
// This library is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public
// License as published by the Free Software Foundation; either
// version 2.1 of the License, or any later version.
//
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// Lesser General Public License for more details.

// You should have received a copy of the GNU Lesser General Public
// License along with this library; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

#ifndef MICAL_GIVE_BS_ELT_HXX_
#define MICAL_GIVE_BS_ELT_HXX_

#include <mical/algorithms/give_bs_elt.hh>

namespace mical {

  template <class A, class B, class C>
  void 
  give_bs_elt(FrameWork<A, B, C>& fw)
  {
    bool stop = false;

    exploration_lattice(fw, 0, 0, stop);
  }
  

  template<class A, class B, class C>
  void 
  exploration_lattice(FrameWork<A, B, C>& fw,
		      hstate_t minor, hstate_t major,
		      bool& stop)
  {
    typedef FrameWork<A, B, C>                           framework_t;
    typedef typename framework_t::view_t::state_iterator state_iterator;
    typedef typename framework_t::view_t::edge_iterator  edge_iterator;
    typedef typename framework_t::block_t                block_t;
    
    if (fw.is_borderset_elt())
      {
	stop = true;
	return;
      }

    if (!fw.is_compatible())
      return;

    if (!fw.is_universal())
      {
	for (state_iterator i = fw.view()->states().begin();
	     i != fw.view()->states().end(); i++)
	  for (state_iterator j = i; j != fw.view()->states().end(); j++) 
	    if (*j != *i)
	      {
		bool flag;

		if (*i < minor)
		  {
		    flag = false;
		  }
		else if (*i >= major)
		  {
		    if (*j >= minor)
		      flag = true;
		    else
		      flag = false;
		  }
		else if (*i == minor)
		  {
		    if (*j < major)
		      flag = false;
		    else
		      flag = true;
		  }
		else if (*i > minor)
		  {
		    if (*j > minor)
		      flag = true;
		    else
		      flag = false;
		  }
		else
		  assert(0); // hummm ... something's wrong

		if (flag)
		  {
		    hstate_t svg1 = *i, svg2 = *j;

		    block_t svg = *fw.partition().find_elt(*j);

		    fw.fusion(svg1, svg2);
		    
		    exploration_lattice(fw, min(svg1, svg2), max(svg1, svg2), stop);
		    
		    if (stop)
		      return;

		    fw.fission(svg1, svg);

		    i = fw.view()->update(svg1);
		    j = fw.view()->update(svg2);
		  }
	      }
      }
    else 
      stop = true;
  }

} // mical

#endif // MICAL_GIVE_BS_ELT_HXX_
