// rpni.hxx
//
// Mical, a regular grammar inference library
// Copyright (C) 2003 Maxime Rey
//
// This library is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public
// License as published by the Free Software Foundation; either
// version 2.1 of the License, or any later version.
//
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// Lesser General Public License for more details.

// You should have received a copy of the GNU Lesser General Public
// License along with this library; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

#ifndef MICAL_RPNI_HXX_
#define MICAL_RPNI_HXX_

#include <mical/algorithms/rpni.hh>

namespace mical {

  template <class A>
  void rpni(FrameWork<search_kind::pta, inference_kind::fixed, A>& fw)
  {
    typedef FrameWork<search_kind::pta, inference_kind::fixed, A> framework_t;
    typedef typename framework_t::partition_t                   partition_t;

    fw.init();

    unsigned i_numstates = fw.view()->states().size();
    
    for (unsigned i = 1; i < i_numstates; i++)
      for (unsigned j = 0; j < i; j++)
	if ((fw.view()->has_state(j)) &&
	    (fw.view()->has_state(i)))
	  {
	    partition_t svg = fw.partition();

	    fw.fusion(i, j);
 	    fw.determin(j);
	    
	    if (fw.is_compatible())
	      j = i;
	    else
	      {
		fw.partition() = svg;
		fw.view()->update();
	      }
	  }
  }


} // mical

#endif // MICAL_RPNI_HXX_
