// order_block.hxx
//
// Mical, a regular grammar inference library
// Copyright (C) 2003 Maxime Rey
//
// This library is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public
// License as published by the Free Software Foundation; either
// version 2.1 of the License, or any later version.
//
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// Lesser General Public License for more details.

// You should have received a copy of the GNU Lesser General Public
// License along with this library; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

#ifndef MICAL_ORDER_BLOCK_HXX_
# define MICAL_ORDER_BLOCK_HXX_

#include <mical/framework/partitions/order_block.hh>

namespace mical {

  namespace min_order 
  {
    template <class Block_>
    bool 
    operator < (const Block_& lhs, const Block_ rhs)
    {
      typename Block_::iterator min_lhs = lhs.begin();
      for (typename Block_::iterator i = lhs.begin();
	   i != lhs.end(); i++)
	if (*min_lhs > *i)
	  min_lhs = i;
      
      typename Block_::iterator min_rhs = rhs.begin();
      for (typename Block_::iterator i = rhs.begin();
	   i != rhs.end(); i++)
	if (*min_rhs > *i)
	  min_rhs = i;	
      
      return *min_lhs < *min_rhs;
    }

  } // min_order

  namespace max_order 
  {
    template <class Block_>
    bool 
    operator < (const Block_& lhs, const Block_ rhs)
    {
      typename Block_::iterator max_lhs = lhs.begin();
      for (typename Block_::iterator i = lhs.begin();
	   i != lhs.end(); i++)
	if (*max_lhs < *i)
	  max_lhs = i;
      
      typename Block_::iterator max_rhs = rhs.begin();
      for (typename Block_::iterator i = rhs.begin();
	   i != rhs.end(); i++)
	if (*max_rhs < *i)
	  max_rhs = i;	
      
      return *max_lhs < *max_rhs;
    }

  } // max_order

} // mical

#endif // MICAL_ORDER_BLOCK_HXX_
