// mca_to_pta.hxx
//
// Mical, a regular grammar inference library
// Copyright (C) 2003 Maxime Rey
//
// This library is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public
// License as published by the Free Software Foundation; either
// version 2.1 of the License, or any later version.
//
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// Lesser General Public License for more details.

// You should have received a copy of the GNU Lesser General Public
// License along with this library; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

#ifndef MICAL_MCA_TO_PTA_HXX_
# define MICAL_MCA_TO_PTA_HXX_

#include <vaucanson/fundamental/element.hh>

#include <mical/samples/kind/summary.hh>

#include <mical/internal/mca_to_pta.hh>
#include <mical/internal/fusion.hh>

#include <set>

namespace mical {

  using std::set;

  template<typename A, typename T>
  Element<A, T>
  pta_from_mca(const Element<A, T>& mca)
  {
    Element<A, T> res = mca;
    res.emancipate();
    
    // mca have an unique initial state
    pta_from_mca_(res, *mca.initial().begin());
    
    return res;
  }
  
  template<typename A, typename T>
  void
  pta_from_mca_(Element<A, T>& a, hstate_t current)
  {
    typedef Element<A, T> automaton_t;

    for (typename automaton_t::monoid_t::alphabet_t::iterator 
	   i = a.series().monoid().alphabet().begin();
	 i != a.series().monoid().alphabet().end(); i++)
      {
	set<hedge_t> delta_edge;
	
	a.letter_deltac(delta_edge, current, *i, delta_kind::edges());

	if (delta_edge.size() > 1)
	  {
	    hstate_t first = a.aim_of(*delta_edge.begin());
	    
	    for (set<hedge_t>::iterator j = ++delta_edge.begin();
		 j != delta_edge.end(); j++)
	      if (a.aim_of(*j) < first)
		first = a.aim_of(*j);

	    for (typename set<hedge_t>::iterator j = ++delta_edge.begin();
		 j != delta_edge.end(); j++)
	      fusion(a, a.aim_of(*j), first);
	  }
      }
    
    set<hstate_t> delta_state;

    a.deltac(delta_state, current, delta_kind::states());

    for (typename set<hstate_t>::iterator i = delta_state.begin();
	 i != delta_state.end(); i++)
      pta_from_mca_(a, *i);
  }

  template<typename A, typename T>
  void
  mca_to_pta(Element<A, T>& a)
  {
    a = pta_from_mca(a);
  }
  
} // mical

#endif // MICAL_MCA_TO_PTA_HXX_
