// compute.hh
//
// Mical, a regular grammar inference library
// Copyright (C) 2003 Maxime Rey
//
// This library is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public
// License as published by the Free Software Foundation; either
// version 2.1 of the License, or any later version.
//
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// Lesser General Public License for more details.

// You should have received a copy of the GNU Lesser General Public
// License along with this library; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

#ifndef MICAL_COMPUTE_HH_
#define MICAL_COMPUTE_HH_

#include <vaucanson/fundamental/element.hh>
#include <vaucanson/automata/concept/automata.hh>

#include <mical/walkers/info.hh>

#include <set>
#include <vector>
#include <map>

namespace mical {
  
  using std::set;
  using std::vector;
  using std::map;
  using namespace vcsn;

 namespace compute_kind {

   struct boolean
   {};
   
   struct state
   {};

   template <typename T>
   struct compute_kind_traits
   {};

   template <>
   struct compute_kind_traits<boolean>
   {
     typedef bool  type_t;
   };
   
   template <>
   struct compute_kind_traits<state>
   {
     typedef hstate_t  type_t;
   };

 } // compute_kind

  template<class A, class T, class Word_> 
  struct Compute_n
  {
    typedef Compute_n<A, T, Word_>        self_t;
    
    typedef InfoDepth<A, T, self_t>       info_t;

    typedef Word_                         word_t;
    
    typedef vector<hstate_t>              delta_state_t;

    Compute_n();

    bool operator ()(Element<A, T> a, 
		     word_t word, 
		     unsigned n, 
		     compute_kind::boolean);

    int  operator ()(Element<A, T> a, 
		     word_t word, 
		     unsigned n, 
		     compute_kind::state);
    
    template<template<class> class Container_>
    void operator ()(Element<A, T> a, 
		     word_t word, 
		     unsigned n, 
		     Container_<hstate_t>& res);

    template<template<class> class Container_>
    void operator ()(Element<A, T> a, 
		     word_t word, 
		     unsigned n, 
		     Container_<hedge_t>& res);
    
    
    void prefix(info_t& info);

    void infix(info_t& info);

    void postfix(info_t& info);

    template<typename Container_>
    void 
    delta(info_t& info, Container_& res, hstate_t from);

  private:

    typename word_t::iterator p;
    word_t str;

    typedef map<hedge_t, int>  path_edges_t;

    path_edges_t       path_edges;

    int                final_state;

    unsigned           count;
    unsigned           max_n;
  };

} // mical

#include <mical/primitives/compute.hxx>

#endif // MICAL_COMPUTE_HH_
