// get_caracteristic.hxx
//
// Mical, a regular grammar inference library
// Copyright (C) 2003 Maxime Rey
//
// This library is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public
// License as published by the Free Software Foundation; either
// version 2.1 of the License, or any later version.
//
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// Lesser General Public License for more details.

// You should have received a copy of the GNU Lesser General Public
// License along with this library; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

#ifndef MICAL_GET_CARACTERISTIC_HXX_
# define MICAL_GET_CARACTERISTIC_HXX_

#include <mical/samples/services/get_caracteristic.hh>

#include <mical/tools/choose.hh>
#include <mical/samples/services/get_short_prefix.hh>
#include <mical/samples/services/get_kernel.hh>

#include <mical/primitives/find_suffix.hh>
#include <mical/primitives/compute_one.hh>
#include <mical/primitives/quotient.hh>

#include <mical/walkers/breadth_search.hh>

namespace mical {

  using namespace vcsn;

  template<class A, class T, class Word_> 
  struct FindCommonSuffix
  {
    typedef FindCommonSuffix<A, T, Word_>   self_t;
    
    typedef InfoBreadth<A, T, self_t>       info_t;

    typedef Word_                           word_t;
    
    typedef vector<hstate_t>                delta_state_t;

    FindCommonSuffix()
    {}
    
    word_t operator ()(const Element<A, T>& a, 
		       hstate_t state,
		       word_t prefix_)
    {
      str = prefix_;
      
      Element<A, T> work = a;
      work.create();

      if (work.states().size())
	breadth_search(work, state, *this, security::responsable_t());
      
      for (unsigned i = 0; i < prefix_.value().size(); i++)
      	str.value().erase(str.begin());
      
      return str;
    }

    void prefix(info_t& info)
    {
      if (!info.first_)
	str = str * info.auto_.label_of(info.edge_).begin()->first;
      
      if (info.auto_.is_final(info.state_))
	{
	  if (!compute_one(info.auto_, str, compute_kind::boolean()))
	    info.stop_ = true;
	}
      else
	{
	  if (compute_one(info.auto_, str, compute_kind::boolean()))
	    info.stop_ = true;
	}
	
    }

    void infix(info_t& info)
    {}

    void postfix(info_t& info)
    {
      if (info.stop_)
	return;

      if (str.value().size() > 0)
	str.value().erase(--str.end());
    }

    template<typename Container_>
    void 
    delta(info_t& info, Container_& res, hstate_t from)
    {
      info.auto_.deltac(res, from, delta_kind::states());
    }

  private:

    word_t str;

    hstate_t state_;
  };

  
  template<class A, class T, class Kind_, class Container_>
  void
  get_caracteristic(const Element<A, T>& a,
		    Presentation<Kind_, Container_, 
		    typename Element<A, T>::monoid_elt_t>& pres)
  {
    typedef typename Element<A, T>::monoid_elt_t word_t;
    
    set<word_t> kernel;
    set<word_t> sp;
        
    get_kernel(a, kernel);
    get_short_prefix(a, sp);
  
    for (typename set<word_t>::iterator i = kernel.begin(); 
	 i != kernel.end(); i++)
      if (compute_one(a, *i, compute_kind::boolean()))
	pres.add_positive(*i);
      else
	pres.add_positive(*i * find_suffix(a, quotient(a, *i)));

    FindCommonSuffix<A, T, word_t> fcs;
    hstate_t state;
    word_t suffix;

    for (typename set<word_t>::iterator i = sp.begin(); i != sp.end(); i++)
      for (typename set<word_t>::iterator j = kernel.begin(); 
	   j != kernel.end(); j++)
	if ((*i != *j) && ((state = quotient(a, *j)) != quotient(a, *i)))
	  {
	    suffix = fcs(a, state, *i);

	    if (compute_one(a, *i * suffix, compute_kind::boolean()))
	      {
		pres.add_positive(*i * suffix);
		pres.add_negative(*j * suffix);
	      }
	    else
	      {
		pres.add_positive(*j * suffix);
		pres.add_negative(*i * suffix);
	      }
	  }
  }



} // mical

#endif // MICAL_GET_CARACTERISTIC_HXX_
