// get_short_prefix.hxx
//
// Mical, a regular grammar inference library
// Copyright (C) 2003 Maxime Rey
//
// This library is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public
// License as published by the Free Software Foundation; either
// version 2.1 of the License, or any later version.
//
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// Lesser General Public License for more details.

// You should have received a copy of the GNU Lesser General Public
// License along with this library; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

#ifndef MICAL_GET_SHORT_PREFIX_HXX_
#define MICAL_GET_SHORT_PREFIX_HXX_

#include <mical/samples/services/get_short_prefix.hh>

#include <vaucanson/fundamental/element.hh>
#include <vaucanson/automata/concept/automata.hh>

#include <mical/walkers/info.hh>
#include <mical/walkers/depth_search.hh>

#include <set>
#include <vector>
#include <algorithm>

namespace mical {

  using namespace vcsn;

  using std::set;
  using std::vector;
  using std::copy;

  template<class A, class T> 
  struct GetShortPrefix
  {
    typedef GetShortPrefix<A, T>                   self_t;
    
    typedef InfoDepth<A, T, self_t>                info_t;

    typedef typename Element<A, T>::monoid_elt_t   word_t;

    typedef vector<hstate_t>                       delta_state_t;

    GetShortPrefix()
    {}
    
    template<class Container_>
    void operator ()(Element<A, T> a, Container_& res)
    {
      str = "";
      container_.insert(str);

      depth_search(a, *this, security::safe_t(1));
      
      copy(container_.begin(), container_.end(),
	   inserter(res, res.begin()));
    }
    
    void prefix(info_t& info)
    {
      if (!info.first_)
	str = str * info.auto_.label_of(info.edge_).begin()->first;
      else
	info.marked_[*info.auto_.initial().begin()]++;
      
      container_.insert(str);
    }

    void infix(info_t& info)
    {}

    void postfix(info_t& info)
    {
      if (str.value().size() > 0)
	str.value().erase(--str.end());
    }

    template<typename Container_>
    void 
    delta(info_t& info, Container_& res, hstate_t from)
    {
      info.auto_.deltac(res, from, delta_kind::states());
    }

  private:

    set<word_t> container_;

    word_t str;
  };


  template <typename A, typename T, typename Container_>
  void 
  get_short_prefix(const Element<A, T>& a, Container_& res)
  {
    GetShortPrefix<A, T> get_sp;

    get_sp(a, res);
  }
  
  template <typename A, typename T, typename Container_>
  void 
  get_short_prefix(const Element<A, T>& a, 
		   FixedSample<Container_,
		   typename Element<A, T>::monoid_elt_t>& res)
  {
    GetShortPrefix<A, T> get_sp;
    
    get_sp(a, res.container()); 
  }

} // mical 

#endif // MICAL_GET_SHORT_PREFIX_HXX_
