// info.hh
//
// Mical, a regular grammar inference library
// Copyright (C) 2003 Maxime Rey
//
// This library is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public
// License as published by the Free Software Foundation; either
// version 2.1 of the License, or any later version.
//
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// Lesser General Public License for more details.

// You should have received a copy of the GNU Lesser General Public
// License along with this library; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

#ifndef MICAL_INFO_HH_
# define MICAL_INFO_HH_

#include <vaucanson/fundamental/element.hh>
#include <vaucanson/automata/concept/automata.hh>

#include <mical/walkers/security.hh>

#include <set>
#include <deque>
#include <vector>
#include <utility>

namespace mical {

  using std::set;
  using std::deque;
  using std::vector;
  using std::map;

  using namespace vcsn;

  template <class A, class T, class Task_> 
  struct Info
  {
    ////////////////////////////////////////////////////////////////////////////
    typedef Task_ task_t;

    typedef Element<A, T> automaton_t;
    ////////////////////////////////////////////////////////////////////////////

    Info();

    // the automaton
    automaton_t auto_;

    // the actual state
    hstate_t state_;

    hstate_t prev_state_;

    // actions and criteria
    task_t task_;

    // true if course is in its first recursive call
    bool first_;

    bool stop_;
  };


  template <class A, class T, class Task_> 
  struct InfoDepth : public Info<A, T, Task_>
  {
    ////////////////////////////////////////////////////////////////////////////
    typedef Info<A, T, Task_>        top_t;
    typedef typename top_t::task_t   task_t; 

    typedef map<hstate_t, int>       marked_t;
    ////////////////////////////////////////////////////////////////////////////

    InfoDepth();

    InfoDepth(unsigned n);
    
    hedge_t  edge_;

    // states which have been already used
    marked_t marked_;

    unsigned nb_cycles_;
  };

  
  template <class A, class T, class Task_> 
  struct InfoBreadth : public Info<A, T, Task_>
  {
    ////////////////////////////////////////////////////////////////////////////
    typedef Info<A, T, Task_>        top_t;
    typedef typename top_t::task_t   task_t; 

    typedef deque<int>               queue_t;

    typedef vector<hstate_t>         vect_t;

    typedef map<unsigned, unsigned>  mappy_t;

    typedef map<hstate_t, int>       marked_t;
    ////////////////////////////////////////////////////////////////////////////

    InfoBreadth();

    InfoBreadth(unsigned n);

    hedge_t  edge_;

    queue_t  queue_;

    marked_t marked_;

    unsigned nb_cycles_;

    vect_t   vect_;

    vector<unsigned> vect_count_;
    
    int      count_;

    int      count_bis_;
  };
  
  template <class A, class T, class Task_, class Order_> 
  struct InfoOrder : public Info<A, T, Task_>
  {
    ////////////////////////////////////////////////////////////////////////////
    typedef Info<A, T, Task_>        top_t;

    typedef set<hstate_t>            marked_t;

    typedef Order_                   order_t;
    typedef typename top_t::task_t   task_t; 
    ////////////////////////////////////////////////////////////////////////////

    InfoOrder();

    // states which have been already used
    marked_t marked_;

    order_t order_;
  };

} // mical

#include <mical/walkers/info.hxx>

#endif // MICAL_INFO_HH_
