// test_framework.cc

#include <vaucanson/fundamental/predefs.hh>
#include <vaucanson/algebra/concrete/free_monoid/predefs.hh>
#include <vaucanson/algebra/concrete/semiring/numerical_semiring.hh>
#include <vaucanson/algebra/concrete/series/polynoms.hh>
#include <vaucanson/automata/concept/automata.hh>
#include <vaucanson/tools/usual.hh>

#include <mical/framework/framework.hh>

#include <mical/algorithms/fusion_determin.hh>

#include <mical/samples/tools/load_sample.hh>

#include <set>


int main(int argc, char** argv)
{
  using namespace std;
  using namespace mical;
  using namespace vcsn;
  using namespace vcsn::algebra;
  using namespace vcsn::algebra::char_letter;
  
  Alphabet a;

  a.insert('a');
  a.insert('b');

  Words free_monoid(a);
  NumericalSemiring ring;
  Series<NumericalSemiring, Words> s(ring, free_monoid);

  typedef Automata auto_t;
  typedef 
    utility::ref< 
    AutomatonImpl<labels_are_series,
    Series<NumericalSemiring, Words>,
    polynom<Word::value_t, bool>,
    ManyLinks<polynom<Word::value_t, bool>, NoTag, NoTag>,
    NoTag,
    map> >
    implem_t;

  typedef Element<auto_t, implem_t> automaton_t;

  

  //////////////////////////////////////////////////////////////////////////////////////////

  typedef   FrameWork<
    search_kind::pta, 
    inference_kind::fixed, 
    framework_traits<automaton_t, set<set<hstate_t> >, set<Word>, inference_kind::fixed> > framework_t;

  framework_t fw("toy.a");
  
  cout << fw.is_compatible() << endl;
  
  cout << fw.view()->edges().size() << endl;
  
  for (framework_t::view_t::edge_iterator i = fw.view()->edges().begin();
       i != fw.view()->edges().end(); i++)
    cout << fw.view()->origin_of(*i) << " - " 
	 << fw.view()->aim_of(*i) << " | " 
	 << fw.view()->label_of(*i).begin()->first << endl;
  
  
  framework_t other(fw);



  for (framework_t::view_t::edge_iterator i = other.view()->edges().begin();
       i != other.view()->edges().end(); i++)
    cout << other.view()->origin_of(*i) << " - " 
	 << other.view()->aim_of(*i) << " | " 
	 << other.view()->label_of(*i).begin()->first << endl;

  other.fusion(0, 1);

  cout << endl;
  for (framework_t::view_t::edge_iterator i = other.view()->edges().begin();
       i != other.view()->edges().end(); i++)
    cout << other.view()->origin_of(*i) << " - " 
	 << other.view()->aim_of(*i) << " | " 
	 << other.view()->label_of(*i).begin()->first << endl;
  
  cout << endl;
  for (framework_t::view_t::edge_iterator i = fw.view()->edges().begin();
       i != fw.view()->edges().end(); i++)
    cout << fw.view()->origin_of(*i) << " - " 
	 << fw.view()->aim_of(*i) << " | " 
	 << fw.view()->label_of(*i).begin()->first << endl;
  
  other.init();

  cout << endl;
  for (framework_t::view_t::edge_iterator i = other.view()->edges().begin();
       i != other.view()->edges().end(); i++)
    cout << other.view()->origin_of(*i) << " - " 
	 << other.view()->aim_of(*i) << " | " 
	 << other.view()->label_of(*i).begin()->first << endl;
  //////////////////////////////////////////////////////////////////////////////////////////
  other.fusion(0, 1);
  other.fission(0, 1);


  
  cout << endl;
  for (framework_t::view_t::edge_iterator i = other.view()->edges().begin();
       i != other.view()->edges().end(); i++)
    cout << other.view()->origin_of(*i) << " - " 
	 << other.view()->aim_of(*i) << " | " 
	 << other.view()->label_of(*i).begin()->first << endl;

  cout << endl;
  cout << other.is_null() << endl;

  cout << endl;
  cout << other.is_lattice_elt() << endl;

  cout << endl;
  cout << other.is_borderset_elt() << endl;

  other.fusion(0, 1);
  other.fusion(0, 2);
  
  other.determin(0);



  cout << "IS UNIVERSAL: " << other.is_universal() << endl;

  Element<auto_t, implem_t> fresh = other.generate();
  
  cout << fresh.series().monoid().alphabet().size() << endl;
  cout << endl;
  
  for (Element<auto_t, implem_t>::edge_iterator i = fresh.edges().begin();
       i != fresh.edges().end(); i++)
    cout << fresh.origin_of(*i) << " - " 
	 << fresh.aim_of(*i) << " | " 
	 << fresh.label_of(*i).begin()->first << endl;
  
  return 0;
}
