// test_viewer.cc

#include <vaucanson/fundamental/predefs.hh>
#include <vaucanson/algebra/concrete/free_monoid/predefs.hh>
#include <vaucanson/algebra/concrete/semiring/numerical_semiring.hh>
#include <vaucanson/algebra/concrete/series/polynoms.hh>
#include <vaucanson/tools/usual.hh>


#include <mical/framework/viewer/view_lattice_elt.hh>

#include <mical/framework/partitions/partitions.hh>

#include <mical/primitives/compute_one.hh>

//#include <mical/framework/framework.hh>
#include <mical/samples/tools/load_sample.hh>
#include <mical/internal/make_pta.hh>
#include <mical/internal/renumerotation.hh>


#include <set>
#include <vector>

int main()
{
  using namespace std;
  using namespace mical;
  using namespace vcsn;
  using namespace vcsn::algebra;
  using namespace vcsn::algebra::char_letter;

  Alphabet a;

  a.insert('a');
  a.insert('b');

  Words free_monoid(a);
  NumericalSemiring ring;
  Series<NumericalSemiring, Words> s(ring, free_monoid);

  typedef Automata auto_t;
  typedef
    utility::ref<
    AutomatonImpl<labels_are_series,
    Series<NumericalSemiring, Words>,
    polynom<Word::value_t, bool>,
    ManyLinks<polynom<Word::value_t, bool>, NoTag, NoTag>,
    NoTag,
    map> >
    implem_t;

  Element<auto_t, implem_t> A, B;

  A.create();
  B.create();

  B.series() = s;
  
  hstate_t n_0 = A.add_state();
  hstate_t n_1 = A.add_state();
  hstate_t n_2 = A.add_state();
  hstate_t n_3 = A.add_state();

  A.set_initial(n_0);

  A.add_letter_edge(n_0, n_1, 'b');
  A.add_letter_edge(n_1, n_2, 'a');

  A.set_final(n_2);

  A.add_letter_edge(n_1, n_3, 'b');

  A.set_final(n_3);


  Partition<partition_traits<set<set<hstate_t> > > > partition;
  
  partition.insert(n_0);
  partition.insert(n_1);
  partition.insert(n_2);
  partition.insert(n_3);
  
  typedef   
    Element<ViewLattice, ImplemViewLattice<Element<auto_t, implem_t>, 
    Partition<partition_traits<set<set<hstate_t> > > > > >
    viewer_t;
  
  viewer_t view(A, partition);

  cout <<  "DISPLAY STATES" << endl;
  for (viewer_t::state_iterator i = view.states().begin();
       i != view.states().end(); i++)
    cout << *i << endl;
  
  cout << endl;

  cout << "DISPLAY EDGES" << endl;
  for (viewer_t::edge_iterator i = view.edges().begin();
       i != view.edges().end(); i++)
    cout << *i << " | " 
	 << view.origin_of(*i) << " - " << view.aim_of(*i) 
	 << " | " << view.label_of(*i).begin()->first
	 << endl;

  cout << "INITIAL" << endl;

  for (viewer_t::state_iterator i = view.initial().begin();
       i != view.initial().end(); i++)
    cout << *i << endl;
  
  cout << "FINAL" << endl;

  for (viewer_t::state_iterator i = view.final().begin();
       i != view.final().end(); i++)
	cout << *i << endl;


  cout << endl;
  cout << "-------- Fusion state 0 and state 1 --------" << endl;

  partition.weld(n_0, n_1);

  view.update();

  cout << "DISPLAY STATES" << endl;
  for (viewer_t::state_iterator i = view.states().begin();
       i != view.states().end(); i++)
    cout << *i << endl;
  
  cout << endl;
  
  cout << "DISPLAY EDGES" << endl;
  for (viewer_t::edge_iterator i = view.edges().begin();
       i != view.edges().end(); i++)
    cout << *i << " | " 
	 << view.origin_of(*i) << " - " << view.aim_of(*i)
	 << " | " << view.label_of(*i).begin()->first
	 << endl;

  cout << "INITIAL" << endl;

  for (viewer_t::state_iterator i = view.initial().begin();
       i != view.initial().end(); i++)
    cout << *i << endl;
  
  cout << "FINAL" << endl;

  for (viewer_t::state_iterator i = view.final().begin();
       i != view.final().end(); i++)
	cout << *i << endl;

  cout << endl;
  cout << "DELTA EDGES of state 0 labelled by b" << endl;

  vector<hedge_t> delta_edge;
  view.deltac(delta_edge, n_0, 'b', delta_kind::edges());
  
  for (vector<hedge_t>::iterator i = delta_edge.begin();
       i != delta_edge.end(); i++)
    cout << *i << " | " 
	 << view.origin_of(*i) << " - " << view.aim_of(*i)
	 << " | " << view.label_of(*i).begin()->first
	 << endl;
  
  cout << endl;
  cout << "RDELTA EDGES of state 0 labelled by b" << endl;

  delta_edge.clear();
  view.rdeltac(delta_edge, n_0, 'b', delta_kind::edges());

  for (vector<hedge_t>::iterator i = delta_edge.begin();
       i != delta_edge.end(); i++)
    cout << *i << " | " 
	 << view.origin_of(*i) << " - " << view.aim_of(*i)
	 << " | " << view.label_of(*i).begin()->first
	 << endl;
  
  cout << endl;
  cout << "DELTA STATES of state 0 labelled by b" << endl;

  vector<hstate_t> delta_state;
  view.deltac(delta_state, n_0, 'b', delta_kind::states());
  
  copy(delta_state.begin(), delta_state.end(), 
       ostream_iterator<hstate_t>(cout, "\n"));


  cout << endl;
  cout << "RDELTA STATES of state 0 labelled by b" << endl;

  delta_state.clear();
  view.rdeltac(delta_state, n_0, 'b', delta_kind::states());
  
  copy(delta_state.begin(), delta_state.end(), 
       ostream_iterator<hstate_t>(cout, "\n"));


  cout << endl;
  cout << "-------- Fusion state 0 and state 3 --------" << endl;
  
  set<hstate_t> block1, block2;

  block1.insert(n_0);
  block1.insert(n_1);

  block2.insert(n_3);

  partition.weld(block1, block2);

  view.update();

  cout << endl;
  cout << "DISPLAY STATES" << endl;
  for (viewer_t::state_iterator i = view.states().begin();
       i != view.states().end(); i++)
    cout << *i << endl;
  
  cout << endl;
  
  cout << "DISPLAY EDGES" << endl;
  for (viewer_t::edge_iterator i = view.edges().begin();
       i != view.edges().end(); i++)
    cout << *i << " | " 
	 << view.origin_of(*i) << " - " << view.aim_of(*i)
	 << " | " << view.label_of(*i).begin()->first
	 << endl;

  cout << "INITIAL" << endl;

  for (viewer_t::state_iterator i = view.initial().begin();
       i != view.initial().end(); i++)
    cout << *i << endl;
  
  cout << "FINAL" << endl;

  for (viewer_t::state_iterator i = view.final().begin();
       i != view.final().end(); i++)
	cout << *i << endl;


  cout << endl;
  cout << "DELTA EDGES of state 0" << endl;

  delta_edge.clear();
  view.deltac(delta_edge, n_0, delta_kind::edges());
  
  for (vector<hedge_t>::iterator i = delta_edge.begin();
       i != delta_edge.end(); i++)
    cout << *i << " | " 
	 << view.origin_of(*i) << " - " << view.aim_of(*i)
	 << " | " << view.label_of(*i).begin()->first
	 << endl;
  

  cout << endl;
  cout << "RDELTA EDGES of state 0" << endl;

  delta_edge.clear();
  view.rdeltac(delta_edge, n_0, delta_kind::edges());

  for (vector<hedge_t>::iterator i = delta_edge.begin();
       i != delta_edge.end(); i++)
    cout << *i << " | " 
	 << view.origin_of(*i) << " - " << view.aim_of(*i)
	 << " | " << view.label_of(*i).begin()->first
	 << endl;
  
  cout << endl;
  cout << "DELTA STATES of state 0" << endl;

  delta_state.clear();
  view.deltac(delta_state, n_0, delta_kind::states());
  
  copy(delta_state.begin(), delta_state.end(), 
       ostream_iterator<hstate_t>(cout, "\n"));


  cout << endl;
  cout << "RDELTA STATES of state 0" << endl;

  delta_state.clear();
  view.rdeltac(delta_state, n_0, delta_kind::states());
  
  copy(delta_state.begin(), delta_state.end(), 
       ostream_iterator<hstate_t>(cout, "\n"));

  

  cout << endl;
  cout << "-------- Separate state 0 and state 1 --------" << endl;
  
  partition.extract(partition.begin(), n_1);

  view.update();

  cout << "DISPLAY STATES" << endl;
  for (viewer_t::state_iterator i = view.states().begin();
       i != view.states().end(); i++)
    cout << *i << endl;
  
  cout << endl;
  
  cout << "DISPLAY EDGES" << endl;
  for (viewer_t::edge_iterator i = view.edges().begin();
       i != view.edges().end(); i++)
    cout << *i << " | " 
	 << view.origin_of(*i) << " - " << view.aim_of(*i)
	 << " | " << view.label_of(*i).begin()->first
	 << endl;

  cout << "INITIAL" << endl;

  for (viewer_t::state_iterator i = view.initial().begin();
       i != view.initial().end(); i++)
    cout << *i << endl;
  
  cout << "FINAL" << endl;

  for (viewer_t::state_iterator i = view.final().begin();
       i != view.final().end(); i++)
	cout << *i << endl;

  cout << "COMPUTE TESTS" << endl;

  cout << compute_one(view, "bb", compute_kind::boolean()) << endl;
  cout << compute_one(view, "ba", compute_kind::boolean()) << endl;
  cout << compute_one(view, "aba", compute_kind::boolean()) << endl;
  cout << compute_one(view, "b", compute_kind::boolean()) << endl;

  ///////////////////////////////////////////////////////////////////////////


  cout << endl;
  cout << "--------- NEW TESTS ------------" << endl;
  cout << endl;
  
  Presentation<inference_kind::fixed, set<Word>, Word> pres;

  load_presentation(pres, "../toy.a");

  make_pta(B, pres);

  standard_renum(B);
     
  Partition<partition_traits<set<set<hstate_t> > > > partition2;

  for (Element<auto_t, implem_t>::state_iterator i = B.states().begin(); 
       i != B.states().end(); i++)
    partition2.insert(*i);

  viewer_t view2(B, partition2);
  
  cout << view2.states().size() << endl;
  cout << view2.edges().size() << endl;
  
  for (viewer_t::edge_iterator i = view2.edges().begin();
       i != view2.edges().end(); i++)
    cout << view2.origin_of(*i) << " - " 
	 << view2.aim_of(*i) << " | " 
	 << view2.label_of(*i).begin()->first << endl;

  return 0;
}
