// test_breadth_search.cc

#include <mical/walkers/depth_search.hh>

#include <vaucanson/fundamental/predefs.hh>
#include <vaucanson/algebra/concrete/free_monoid/predefs.hh>
#include <vaucanson/algebra/concrete/semiring/numerical_semiring.hh>
#include <vaucanson/algebra/concrete/series/polynoms.hh>
#include <vaucanson/tools/usual.hh>

#include <vaucanson/automata/concept/automata.hh>
#include <vaucanson/automata/concept/automaton_impl.hh>
#include <vaucanson/automata/concrete/manylinks.hh>

#include <set>
#include <vector>
#include <string>
#include <iostream>

using namespace mical;
using std::cout;
using std::endl;

//////////////////////////////////////////////////////////////////////
template<class A, class T> 
struct DepthPrint
{
  typedef DepthPrint<A, T>          self_t;
    
  typedef InfoDepth<A, T, self_t>   info_t;

  typedef std::vector<hstate_t>     delta_state_t;

  DepthPrint()
  {
  }

  void
  operator ()(const Element<A, T>& a, unsigned sec)
  {
    depth_search(a, *this, security::safe_t(sec));
  }
    
  void prefix(info_t& info)
  {
    cout << "PREFIX" << endl;
    cout << "Actual State: " << info.state_ 
	 << " | Previous State:" << info.prev_state_
	 << " | Actual Edge: " << info.edge_
	 << " | first call? " << info.first_ << endl;
  }

  void infix(info_t& info)
  {
    cout << "INFIX" << endl;
    cout << "Actual State: " << info.state_ 
	 << " | Previous State:" << info.prev_state_
	 << " | Actual Edge: " << info.edge_
	 << " | first call? " << info.first_ << endl;
  }

  void postfix(info_t& info)
  {
    cout << "POSTFIX" << endl;
    cout << "Actual State: " << info.state_ 
	 << " | Previous State:" << info.prev_state_
	 << " | Actual Edge: " << info.edge_
	 << " | first call? " << info.first_ << endl;
  }

  template<typename Container_>
  void 
  delta(info_t& info, Container_& res, hstate_t from)
  {
    info.auto_.deltac(res, from, delta_kind::states());
  }

};
////////////////////////////////////////////////////////////////////


int main(int argc, char** argv)
{
  using namespace std;
  using namespace mical;
  using namespace vcsn;
  using namespace vcsn::algebra;
  using namespace vcsn::algebra::char_letter;
  using std::set;
  using std::string;
  
  Alphabet a;

  a.insert('a');
  a.insert('b');
  
  Words free_monoid(a);
  NumericalSemiring ring;
  Series<NumericalSemiring, Words> s(ring, free_monoid);

  typedef Automata auto_t;
  typedef 
    utility::ref< 
    AutomatonImpl<labels_are_series,
    Series<NumericalSemiring, Words>,
    polynom<Word::value_t, bool>,
    ManyLinks<polynom<Word::value_t, bool>, NoTag, NoTag>,
    NoTag,
    map> >
    implem_t;
  
  Element<auto_t, implem_t> A;

  A.create();

  hstate_t n_0 = A.add_state();
  hstate_t n_1 = A.add_state();
  hstate_t n_2 = A.add_state();
  hstate_t n_3 = A.add_state();
  hstate_t n_4 = A.add_state();
  hstate_t n_5 = A.add_state();

  A.set_initial(n_0);
  A.set_final(n_3);

  A.add_letter_edge(n_0, n_1, 'b');
  A.add_letter_edge(n_1, n_2, 'a');

  A.add_letter_edge(n_5, n_3, 'b');

  A.add_letter_edge(n_1, n_4, 'a');
  A.add_letter_edge(n_1, n_5, 'a');
  A.add_letter_edge(n_5, n_1, 'a');

  SAVE_AUTOMATON_DOT_SIMPLE("automaton", A);

  DepthPrint<auto_t, implem_t> depth;

  cout << "First Search" << endl;
  depth(A, 1);

  cout << "Second Search" << endl;
  depth(A, 2);

  return 0;
}
