/* Converts SEQUITUR rules to BISON rules  */

%{
#define YYSTYPE double
#include <math.h>
#include <stdio.h>
%}

%token NUM
%token CHA

%% /* Grammar rules and actions follow */

input:    /* empty */
        | input line
;
  
line:     '\n'
        | rule '\n'  { printf (";\n"); }
;

rule:   left '-' '>' right             
;

left:   NUM                            { printf("%s%d: ", "S",(int)$1);   }
;

right:     right num_cha
        |  num_cha
;

num_cha:   NUM                          { printf(" %s%d ", "S",(int)$1);   }
        |  CHA                          { if((int)$1 != '|') printf(" \'%c\' ", (int)$1); else printf(" | ");   }
;

%%



/* Lexical analyzer returns a double floating point 
   number on the stack and the token NUM, or the ASCII
   character read if not a number.  Skips all blanks
   and tabs, returns 0 for EOF. */

#include <ctype.h>
#include <stdio.h>

yylex ()
{
  int c;

  /* skip white space  */
  while ((c = getchar ()) == ' ' || c == '\t')  
    ;
  /* process numbers   */
  if (c == '.' || isdigit (c))                
    {
      ungetc (c, stdin);
      scanf ("%lf", &yylval);
      return NUM;
    }
  /* return end-of-file  */
  if (c == EOF)                            
    return 0;

  if (c == '\n')
    return c;

  if (c == '-')
    return c;

  if (c == '>') {
    return c;
  }

  /* return single chars */
  yylval = c;
  return CHA;                                
}

main ()
{
  yyparse ();
}


yyerror (s)  /* Called by yyparse on error */
     char *s;
{
  printf ("...There was as error converting from sequitur to bison file format\n");
}
  
