(***********************************************************
*                                                          *
*      FUNGI - Fast UNsupervised Grammar Induction         *
*                                                          *
*   Originator: Prachya Boonkwan [1], [2]                  *
*               [1] School of Informatics,                 *
*                   University of Edinburgh, UK            *
*               [2] National Electronics and Computer      *
*                   Technology Center (NECTEC), Thailand   *
*                                                          *
*   Copyright (C) 2011 Prachya Boonkwan                    *
*   <p.boonkwan@sms.ed.ac.uk, kaamanita@hotmail.com>       *
*                                                          *
*   This library is free software; you can redistribute    *
*   it and/or modify it under the terms of the GNU         *
*   Lesser General Public License as published by the      *
*   Free Software Foundation; either version 2.1 of the    *
*   License, or (at your option) any later version.        *
*                                                          *
*   This library is distributed in the hope that it will   *
*   be useful, but WITHOUT ANY WARRANTY; without even      *
*   the implied warranty of MERCHANTABILITY or FITNESS     *
*   FOR A PARTICULAR PURPOSE.  See the GNU Lesser          *
*   General Public License for more details.               *
*                                                          *
*   You should have received a copy of the GNU Lesser      *
*   General Public License along with this library; if     *
*   not, write to the Free Software Foundation, Inc.,      *
*   59 Temple Place, Suite 330, Boston, MA  02111-1307     *
*   USA.                                                   *
*                                                          *
***********************************************************)

type inoutmode_t = StandardInOut | VBInOut

type ann_t = NoAnn | DetAnn | SkewedDetAnn | StructAnn

type annmode_t =
    NoAnnealing
  | DetAnnealing of float * float
  | SkewedDetAnnealing of float * float
  | StructAnnealing of float * float * float

module AnnealRunner :
  sig
    type syncat_t = Cg.Syncat.t
    val score_prior :
      Deriv.Deriv.t -> Dict.Dict.t -> Cg.Syncat.t -> 'a -> 'a -> 'a -> 'a
    val init_ruleprior :
      int ->
      float ->
      Dict.Dict.t ->
      Cg.Syncat.t ->
      float ->
      float -> float -> Deriv.DerivGenModel.t -> Deriv.DerivGenModel.t
    val init_probs :
      Chartdb.ChartDbWrapper.t ->
      int ->
      float ->
      Models.probflag_t list ->
      Dict.Dict.t ->
      Cg.Syncat.t ->
      float ->
      float ->
      float ->
      bool -> bool ->
      Deriv.DerivGenModel.t * Deriv.DerivGenModel.t * Deriv.DerivGenModel.t * Hmm.fwdbwd_t
    val run_anneal :
      inoutmode_t ->
      bool ->
      annmode_t ->
      Anneal.SkewedDetParam.genmodel_t ->
      bool -> bool ->
      int ->
      float ->
      int ->
      float ->
      Anneal.EmAnnealer.genmodel_t ->
      Anneal.EmAnnealer.genmodel_t ->
      Anneal.EmAnnealer.syncat_t -> 
      Anneal.EmAnnealer.fn_get_chart_t -> int -> Hmm.fwdbwd_t -> unit
    val train :
      (int ->
       float ->
       int ->
       float ->
       Anneal.EmAnnealer.genmodel_t ->
       Anneal.EmAnnealer.genmodel_t ->
       Anneal.EmAnnealer.syncat_t ->
       Anneal.EmAnnealer.fn_get_chart_t -> int -> Hmm.fwdbwd_t -> unit) ->
       int ->
       float ->
       int ->
       float -> Anneal.EmAnnealer.genmodel_t -> Anneal.EmAnnealer.genmodel_t -> Anneal.EmAnnealer.syncat_t -> Chartdb.ChartDbWrapper.t -> Hmm.fwdbwd_t -> unit
    val save_prob : Deriv.DerivGenModel.t -> string -> unit
    val compute :
      Models.probflag_t list ->
      inoutmode_t ->
      bool ->
      annmode_t ->
      float ->
      int ->
      float ->
      Anneal.EmAnnealer.syncat_t ->
      Dict.Dict.t -> float -> float -> float -> int -> 
      string list -> string -> string ->
      bool -> bool -> bool -> unit
  end

val set_probflags : (Models.probflag_t list ref * string list ref) -> string -> unit
val set_inoutmode : (inoutmode_t ref * string ref) -> string -> unit
val set_annmode : (ann_t ref * string ref) -> string -> unit
val set_start : Cg.Syncat.t ref -> string -> unit
val conv_annparam : ann_t -> float -> float -> float -> annmode_t
