(** CKY Parsing Algorithm *)

(***********************************************************
*                                                          *
*      FUNGI - Fast UNsupervised Grammar Induction         *
*                                                          *
*   Originator: Prachya Boonkwan [1], [2]                  *
*               [1] School of Informatics,                 *
*                   University of Edinburgh, UK            *
*               [2] National Electronics and Computer      *
*                   Technology Center (NECTEC), Thailand   *
*                                                          *
*   Copyright (C) 2011 Prachya Boonkwan                    *
*   <p.boonkwan@sms.ed.ac.uk, kaamanita@hotmail.com>       *
*                                                          *
*   This library is free software; you can redistribute    *
*   it and/or modify it under the terms of the GNU         *
*   Lesser General Public License as published by the      *
*   Free Software Foundation; either version 2.1 of the    *
*   License, or (at your option) any later version.        *
*                                                          *
*   This library is distributed in the hope that it will   *
*   be useful, but WITHOUT ANY WARRANTY; without even      *
*   the implied warranty of MERCHANTABILITY or FITNESS     *
*   FOR A PARTICULAR PURPOSE.  See the GNU Lesser          *
*   General Public License for more details.               *
*                                                          *
*   You should have received a copy of the GNU Lesser      *
*   General Public License along with this library; if     *
*   not, write to the Free Software Foundation, Inc.,      *
*   59 Temple Place, Suite 330, Boston, MA  02111-1307     *
*   USA.                                                   *
*                                                          *
***********************************************************)

(** CKY Parsing Algorithm Module *)
module CKY :
sig
    type syncat_t = Cg.Syncat.t

    (** Span *)
    type span_t = Chart.Chart.span_t
    
    (** Chart *)
    type chart_t = Chart.Chart.t
    
    (** Lexical item *)
    type word_t = Dict.Dict.word_t
    
    (** Dictionary *)
    type dict_t = Dict.Dict.t
    
    (** CKY parser *)
    type t
    
    (** Create a parser with a sentence and a dictionary. *)
    val create : word_t list -> dict_t -> syncat_t -> t

    val change_transf : t -> Cg.CG.transf_t -> unit

    val to_string : t -> string
    
    (** Initialize the chart with the sentence. *)
    val init_chart : ?enum:bool -> t -> unit
    
    (** Apply functional application to two consecutive edges. *)
    val combine_edges : t -> span_t -> unit
    
    (** Apply the coordination rule to three consecutive edges. *)
    val conjoin_edges : t -> span_t -> unit
    
    (** Build the chart from the initialized one with functional 
        application and the coordination rule. *)
    val build_chart : t -> unit
    
    (** Parse a sentence with a dictionary. *)
    val parse : 
        ?transf:Cg.CG.transf_t -> ?depstr:Dep.DepStruct.t option -> 
        word_t list -> dict_t -> syncat_t -> chart_t
end
