(***********************************************************
*                                                          *
*      FUNGI - Fast UNsupervised Grammar Induction         *
*                                                          *
*   Originator: Prachya Boonkwan [1], [2]                  *
*               [1] School of Informatics,                 *
*                   University of Edinburgh, UK            *
*               [2] National Electronics and Computer      *
*                   Technology Center (NECTEC), Thailand   *
*                                                          *
*   Copyright (C) 2011 Prachya Boonkwan                    *
*   <p.boonkwan@sms.ed.ac.uk, kaamanita@hotmail.com>       *
*                                                          *
*   This library is free software; you can redistribute    *
*   it and/or modify it under the terms of the GNU         *
*   Lesser General Public License as published by the      *
*   Free Software Foundation; either version 2.1 of the    *
*   License, or (at your option) any later version.        *
*                                                          *
*   This library is distributed in the hope that it will   *
*   be useful, but WITHOUT ANY WARRANTY; without even      *
*   the implied warranty of MERCHANTABILITY or FITNESS     *
*   FOR A PARTICULAR PURPOSE.  See the GNU Lesser          *
*   General Public License for more details.               *
*                                                          *
*   You should have received a copy of the GNU Lesser      *
*   General Public License along with this library; if     *
*   not, write to the Free Software Foundation, Inc.,      *
*   59 Temple Place, Suite 330, Boston, MA  02111-1307     *
*   USA.                                                   *
*                                                          *
***********************************************************)

open Cg
open Dict
open Dictloader
open Models
open Annealrunner
open Printf

let main =
    let probflags = ref []
    and inoutmode = ref StandardInOut
    and babystep = ref false
    and leapfrogthr = ref 15
    and annmode = ref NoAnn
    and diffthr = ref 1e-6
    and rndthr = ref 5
    and epsilon = ref 1e-25
    and start = ref (Syncat.create_atom "ROOT")
    and defprior = ref 1.0
    and knownderivprior = ref 5.0
    and startderivprior = ref 10000.0
    and initparam = ref 0.0
    and epoch = ref 0.0
    and finalparam = ref 0.0
    and servermode = ref false
    and clientmode_probinit = ref true
    and clientmode_estep = ref false
    and clientpath = ref "/usr/local/bin"
    and hostname = ref "localhost"
    and port_probinit = ref 12345
    and port_estep = ref 12346
    and fnames = ref []
    and ofname = ref "" 
    and ohmm = ref "" in
    let args = [ 
        ("--host",
            Arg.Set_string hostname,
            "[hostname]   Set the Map/Reduce server's hostname");
        ("--port",
            Arg.Set_int port_probinit,
            "[int]   Set the Map/Reduce server's port");
        ("--clientpath",
            Arg.Set_string clientpath,
            "[path/to/bin]   Set the path of client programs (default = /usr/local/bin)")
    ]
    and fn_anon = (fun f -> fnames := !fnames @ [f])
    and usagemsg =
        "Client program for probability table initialization.\n\n"
        ^ (sprintf "Usage: %s" Sys.argv.(0)) in
    Arg.parse args fn_anon usagemsg;
    let annparam = conv_annparam !annmode !initparam !epoch !finalparam in
    Utils.set_clientpath !clientpath;
    let dict = Dict.create 1 Dict.NoEnum in
    if not !clientmode_probinit && not !clientmode_estep then begin
        fprintf stderr "[1mTraining the model on [%s].[0m\n>> " (String.concat ", " !fnames);
        flush stderr
    end else ();
    leapfrogthr := !leapfrogthr + 1;
    let starttime = Unix.time () in
    Utils.set_hostname !hostname;
    Utils.set_port_probinit !port_probinit;
    Utils.set_port_estep !port_estep;
    AnnealRunner.compute 
        !probflags !inoutmode !babystep annparam !diffthr !rndthr !epsilon 
        !start dict !defprior !knownderivprior !startderivprior !leapfrogthr
        !fnames !ofname !ohmm !servermode !clientmode_probinit !clientmode_estep;
    let endtime = Unix.time () in
    if not !clientmode_probinit && not !clientmode_estep then begin
        fprintf stderr ">> Done\n";
        flush stderr;
        let duration = endtime -. starttime in
        fprintf stderr ">> Total runtime: %.2f mins\n" (duration /. 60.0)
    end else ();

