(***********************************************************
*                                                          *
*      FUNGI - Fast UNsupervised Grammar Induction         *
*                                                          *
*   Originator: Prachya Boonkwan [1], [2]                  *
*               [1] School of Informatics,                 *
*                   University of Edinburgh, UK            *
*               [2] National Electronics and Computer      *
*                   Technology Center (NECTEC), Thailand   *
*                                                          *
*   Copyright (C) 2011 Prachya Boonkwan                    *
*   <p.boonkwan@sms.ed.ac.uk, kaamanita@hotmail.com>       *
*                                                          *
*   This library is free software; you can redistribute    *
*   it and/or modify it under the terms of the GNU         *
*   Lesser General Public License as published by the      *
*   Free Software Foundation; either version 2.1 of the    *
*   License, or (at your option) any later version.        *
*                                                          *
*   This library is distributed in the hope that it will   *
*   be useful, but WITHOUT ANY WARRANTY; without even      *
*   the implied warranty of MERCHANTABILITY or FITNESS     *
*   FOR A PARTICULAR PURPOSE.  See the GNU Lesser          *
*   General Public License for more details.               *
*                                                          *
*   You should have received a copy of the GNU Lesser      *
*   General Public License along with this library; if     *
*   not, write to the Free Software Foundation, Inc.,      *
*   59 Temple Place, Suite 330, Boston, MA  02111-1307     *
*   USA.                                                   *
*                                                          *
***********************************************************)

open Cg
open Dict
open Dictloader
open Models
open Sampler
open Printf

let main =
    let clientpath = ref "/usr/local/bin"
    and hostname = ref "localhost"
    and port = ref 12345 in
    let args = [ 
        ("--host",
            Arg.Set_string hostname,
            "[hostname]   Set the Map/Reduce server's hostname");
        ("--port",
            Arg.Set_int port,
            "[int]   Set the Map/Reduce server's port");
        ("--clientpath",
            Arg.Set_string clientpath,
            "[path/to/bin]   Set the path of client programs (default = /usr/local/bin)")
    ]
    and fn_anon = (fun _ -> ())
    and usagemsg =
        "Client program for the Metropolis-Hastings Sampler.\n\n"
        ^ (sprintf "Usage: %s [options]" Sys.argv.(0)) in
    Arg.parse args fn_anon usagemsg;
    Utils.set_clientpath !clientpath;
    Utils.set_hostname !hostname;
    Utils.set_port_estep !port;
    Sampler.compute_onernd_client ()
