type syncat_t = Cg.Syncat.t
type word_t = string
type sent_t = word_t list
type time_t = int
type chart_t = Chart.Chart.t

module TransInst :
  sig
    type t
    val create : syncat_t -> syncat_t -> t
    val get_src : t -> syncat_t
    val get_dest : t -> syncat_t
    val equal : t -> t -> bool
    val hash : t -> int
    val to_string : t -> string
  end
module EmisInst :
  sig
    type t
    val create : syncat_t -> word_t -> t
    val get_syncat : t -> syncat_t
    val get_word : t -> word_t
    val equal : t -> t -> bool
    val hash : t -> int
    val to_string : t -> string
  end
module InitInst :
  sig
    type t
    val create : syncat_t -> t
    val get_syncat : t -> syncat_t
    val equal : t -> t -> bool
    val hash : t -> int
    val to_string : t -> string
  end
module CatInst :
  sig
    type t
    val create : syncat_t -> t
    val equal : t -> t -> bool
    val hash : t -> int
    val to_string : t -> string
  end
module InitProb :
sig
    type inst_t = InitInst.t
    type t
    type list_t = (inst_t * float) list
    val create : int -> float -> t
    val get_epsilon : t -> float
    val mem : t -> inst_t -> bool
    val length : t -> int
    val get_prob : t -> inst_t -> float
    val get_smooth_prob : t -> inst_t -> float
    val set_prob : t -> inst_t -> float -> unit
    val acc_prob : t -> inst_t -> float -> unit
    val iter : t -> (inst_t -> float -> unit) -> unit
    val to_string : t -> string
    val serialize : t -> list_t
end
module TransProb :
  sig
    type inst_t = TransInst.t
    type t
    type list_t = (inst_t * float) list
    val create : int -> float -> t
    val get_epsilon : t -> float
    val mem : t -> inst_t -> bool
    val length : t -> int
    val get_prob : t -> inst_t -> float
    val get_smooth_prob : t -> inst_t -> float
    val set_prob : t -> inst_t -> float -> unit
    val acc_prob : t -> inst_t -> float -> unit
    val iter : t -> (inst_t -> float -> unit) -> unit
    val to_string : t -> string
    val serialize : t -> list_t
  end
module EmisProb :
  sig
    type inst_t = EmisInst.t
    type t
    type list_t = (inst_t * float) list
    val create : int -> float -> t
    val get_epsilon : t -> float
    val mem : t -> inst_t -> bool
    val length : t -> int
    val get_prob : t -> inst_t -> float
    val get_smooth_prob : t -> inst_t -> float
    val set_prob : t -> inst_t -> float -> unit
    val acc_prob : t -> inst_t -> float -> unit
    val iter : t -> (inst_t -> float -> unit) -> unit
    val to_string : t -> string
    val serialize : t -> list_t
  end
module CatProb :
  sig
    type inst_t = CatInst.t
    type t
    type list_t = (inst_t * float) list
    val create : int -> float -> t
    val get_epsilon : t -> float
    val mem : t -> inst_t -> bool
    val length : t -> int
    val get_prob : t -> inst_t -> float
    val get_smooth_prob : t -> inst_t -> float
    val set_prob : t -> inst_t -> float -> unit
    val acc_prob : t -> inst_t -> float -> unit
    val iter : t -> (inst_t -> float -> unit) -> unit
    val to_string : t -> string
    val serialize : t -> list_t
  end

module VisitedInst : sig
    type t
    val create : t -> t
    val create_trans : syncat_t -> syncat_t -> t
    val create_emis : syncat_t -> word_t -> t
    val create_init : syncat_t -> t
    val equal : t -> t -> bool
    val hash : t -> int
    val to_string : t -> string
end
module VisitedProb :
  sig
    type inst_t = VisitedInst.t
    type t
    type list_t = (inst_t * float) list
    val create : int -> float -> t
    val get_epsilon : t -> float
    val mem : t -> inst_t -> bool
    val length : t -> int
    val get_prob : t -> inst_t -> float
    val get_smooth_prob : t -> inst_t -> float
    val set_prob : t -> inst_t -> float -> unit
    val acc_prob : t -> inst_t -> float -> unit
    val iter : t -> (inst_t -> float -> unit) -> unit
    val to_string : t -> string
    val serialize : t -> list_t
  end

type fwdbwd_t
type fwdbwdserl_t

val create : float -> fwdbwd_t
val to_string : fwdbwd_t -> string
val print : fwdbwd_t -> unit
val serialize : fwdbwd_t -> fwdbwdserl_t
val deserialize : fwdbwdserl_t -> fwdbwd_t
val merge : fwdbwd_t -> fwdbwdserl_t -> unit
val get_epsilon : fwdbwd_t -> float
val get_visited : fwdbwd_t -> VisitedProb.t
val get_initprob : fwdbwd_t -> InitProb.t
val get_transprob : fwdbwd_t -> TransProb.t
val get_emisprob : fwdbwd_t -> EmisProb.t
val get_initexp : fwdbwd_t -> InitProb.t
val get_transexp : fwdbwd_t -> TransProb.t
val get_emisexp : fwdbwd_t -> EmisProb.t
val get_catexp : fwdbwd_t -> CatProb.t
val get_totalcatexp : fwdbwd_t -> float ref
val init_onestep : fwdbwd_t -> chart_t -> unit
val train_onestep : fwdbwd_t -> chart_t -> unit
val estimate : fwdbwd_t -> unit
val from_file : string -> float -> fwdbwd_t
val to_file : fwdbwd_t -> string -> unit
