(***********************************************************
*                                                          *
*      FUNGI - Fast UNsupervised Grammar Induction         *
*                                                          *
*   Originator: Prachya Boonkwan [1], [2]                  *
*               [1] School of Informatics,                 *
*                   University of Edinburgh, UK            *
*               [2] National Electronics and Computer      *
*                   Technology Center (NECTEC), Thailand   *
*                                                          *
*   Copyright (C) 2011 Prachya Boonkwan                    *
*   <p.boonkwan@sms.ed.ac.uk, kaamanita@hotmail.com>       *
*                                                          *
*   This library is free software; you can redistribute    *
*   it and/or modify it under the terms of the GNU         *
*   Lesser General Public License as published by the      *
*   Free Software Foundation; either version 2.1 of the    *
*   License, or (at your option) any later version.        *
*                                                          *
*   This library is distributed in the hope that it will   *
*   be useful, but WITHOUT ANY WARRANTY; without even      *
*   the implied warranty of MERCHANTABILITY or FITNESS     *
*   FOR A PARTICULAR PURPOSE.  See the GNU Lesser          *
*   General Public License for more details.               *
*                                                          *
*   You should have received a copy of the GNU Lesser      *
*   General Public License along with this library; if     *
*   not, write to the Free Software Foundation, Inc.,      *
*   59 Temple Place, Suite 330, Boston, MA  02111-1307     *
*   USA.                                                   *
*                                                          *
***********************************************************)

open Cg
open Parserrunner
open Printf

let main =
    Utils.print_version ();
    let dict = ref None
    and start = ref (Syncat.create_atom "ROOT")
    and golddeps = ref []
    and conlldeps = ref []
    and output = ref None 
    and fnames = ref [] in
    let args = [
        ("--dict", 
            Arg.String (fun d -> dict := Some d), 
            "[somedict.txt]   Specify the dictionary for the parser (required)");
        ("--golddeps",
            Arg.String (fun gd -> golddeps := !golddeps @ [gd]),
            "[golddeps.txt]   Specify a gold-standard dependency file (can be specified more than once). These dependency files are used for constraining the CKY parser. Select either this option or --conlldeps.");
        ("--conlldeps",
            Arg.String (fun cd -> conlldeps := !conlldeps @ [cd]),
            "[conlldeps.txt]   Specify a gold-standard CoNLL2006-formatted dependency file (can be specified more than once). These dependency files are used for constraining the CKY parser. Select either this option or --golddeps.");
        ("--output", 
            Arg.String (fun o -> output := Some o), 
            "[output.chartdb]   Specify the output chart database (required)")
    ]
    and fn_anon = (fun f -> fnames := !fnames @ [f]) 
    and usagemsg = 
        "Parse the text files line by line with the CKY Algorithm and the specified dictionary.\n\n" 
        ^ (sprintf "Usage: %s [options] text1 text2 ..." Sys.argv.(0)) in
    Arg.parse args fn_anon usagemsg;
    if !golddeps <> [] && !conlldeps <> [] then
        Arg.usage args usagemsg
    else ();
    match !dict, !output with
        Some dictfname, Some cdbfname ->
        begin
            fprintf stderr "[1mParsing the input [%s] with the grammar %s.[0m\n"
                (String.concat ", " !fnames) dictfname;
            flush stderr;
            let prs = ParserRunner.create dictfname cdbfname !fnames !start !golddeps !conlldeps in
            ParserRunner.run prs
        end
      | _, _ -> Arg.usage args usagemsg
