(** Hierarchical Scale Display. *)

(***********************************************************
*                                                          *
*      FUNGI - Fast UNsupervised Grammar Induction         *
*                                                          *
*   Originator: Prachya Boonkwan [1], [2]                  *
*               [1] School of Informatics,                 *
*                   University of Edinburgh, UK            *
*               [2] National Electronics and Computer      *
*                   Technology Center (NECTEC), Thailand   *
*                                                          *
*   Copyright (C) 2011 Prachya Boonkwan                    *
*   <p.boonkwan@sms.ed.ac.uk, kaamanita@hotmail.com>       *
*                                                          *
*   This library is free software; you can redistribute    *
*   it and/or modify it under the terms of the GNU         *
*   Lesser General Public License as published by the      *
*   Free Software Foundation; either version 2.1 of the    *
*   License, or (at your option) any later version.        *
*                                                          *
*   This library is distributed in the hope that it will   *
*   be useful, but WITHOUT ANY WARRANTY; without even      *
*   the implied warranty of MERCHANTABILITY or FITNESS     *
*   FOR A PARTICULAR PURPOSE.  See the GNU Lesser          *
*   General Public License for more details.               *
*                                                          *
*   You should have received a copy of the GNU Lesser      *
*   General Public License along with this library; if     *
*   not, write to the Free Software Foundation, Inc.,      *
*   59 Temple Place, Suite 330, Boston, MA  02111-1307     *
*   USA.                                                   *
*                                                          *
***********************************************************)

(** Types of display scales. *)
type disp_t =
    Guage of int
  | Number of int
  | Percentage of int
  | NumberNoMax
  | WindMill of int
  | GuageWithWindMill of int * int
  | GuageWithNumber of int
  | GuageWithPercentage of int
  | PercentageWithWindMill of int * int
  | StaticInfix
  | MapReduce
  | MapCombineReduce
  | MapReduceWithWindMill of int

(** [add disp p s] pushes a display scale of type [disp] with
    a leading string [p] and a following string [s] to the top of the
    stack. *)
val add : disp_t -> string -> string -> unit

(** [discard ()] removes the topmost display scale from the stack. *)
val discard : unit -> unit

(** [inc ()] increases the counter of the topmost scale. *)
val inc : unit -> unit

val inc2 : unit -> unit

val inc3 : unit -> unit

(** [display ()] displays the hierarchical scale display on the standard
    error channel. *)
val display : unit -> unit

(** [inc_and_display ()] increases the counter of the topmost scale and 
    displays the scale on the standard error channel. *)
val inc_and_display : unit -> unit

val inc2_and_display : unit -> unit

val inc3_and_display : unit -> unit

(** [print s] prints a text [s] to the standard error channel. *)
val print : string -> unit

(** [clearline ()] clears the current line. *)
val clearline : unit -> unit

(** [redisplay ()] displays the hierarchical scale display on the standard
    error channel. *)
val redisplay : unit -> unit
