(***********************************************************
*                                                          *
*      FUNGI - Fast UNsupervised Grammar Induction         *
*                                                          *
*   Originator: Prachya Boonkwan [1], [2]                  *
*               [1] School of Informatics,                 *
*                   University of Edinburgh, UK            *
*               [2] National Electronics and Computer      *
*                   Technology Center (NECTEC), Thailand   *
*                                                          *
*   Copyright (C) 2011 Prachya Boonkwan                    *
*   <p.boonkwan@sms.ed.ac.uk, kaamanita@hotmail.com>       *
*                                                          *
*   This library is free software; you can redistribute    *
*   it and/or modify it under the terms of the GNU         *
*   Lesser General Public License as published by the      *
*   Free Software Foundation; either version 2.1 of the    *
*   License, or (at your option) any later version.        *
*                                                          *
*   This library is distributed in the hope that it will   *
*   be useful, but WITHOUT ANY WARRANTY; without even      *
*   the implied warranty of MERCHANTABILITY or FITNESS     *
*   FOR A PARTICULAR PURPOSE.  See the GNU Lesser          *
*   General Public License for more details.               *
*                                                          *
*   You should have received a copy of the GNU Lesser      *
*   General Public License along with this library; if     *
*   not, write to the Free Software Foundation, Inc.,      *
*   59 Temple Place, Suite 330, Boston, MA  02111-1307     *
*   USA.                                                   *
*                                                          *
***********************************************************)

open Cg
open Chart
open Chartdb
open Deriv
open Printf

let totalcnt = ref 0

let succcnt = ref 0

let list_chart_word syncat span headpos word =
    let (begpos, endpos) = span in
    let deriv = conv_word_to_deriv syncat word in
    printf "(%d, %d) : %s\n" begpos endpos (Deriv.to_string deriv)

let list_chart_dtrs syncat span headpos word depdir dtrargs =
    let (begpos, endpos) = span in
    let deriv = conv_dtrargs_to_deriv syncat word depdir dtrargs in
    printf "(%d, %d) : %s\n" begpos endpos (Deriv.to_string deriv)

let list_chart chart =
    let n = List.length (Chart.get_sentence chart) in
    Chart.iter_derivs chart Chart.BottomUp 
        list_chart_word list_chart_dtrs (0, n - 1)

let showchartdb showchart showderivs showunparsable showderivcnt fname =
    if showchart || showunparsable || showderivcnt then
        printf "Chart database: %s\n\n" fname
    else ();
    let chartdb = (* ChartDb.opendb fname [Mdb.ReadOnly] *) ChartDb.from_file fname in
    ChartDb.iter
        (fun id chart ->
            if showderivcnt then
                printf "<Chart %d>\nSentence = %s\nDeriv no = %d\n\n" 
                    id 
                    (String.concat " "(Chart.get_sentence chart))
                    (Chart.get_derivcnt chart)
            else if showunparsable && not (Chart.is_success chart) then
                printf "<Chart %d>\nSentence = %s\n\n" id (String.concat " "(Chart.get_sentence chart))
            else if showchart then
                printf "<Chart %d>\n%s\n\n" id (Chart.to_string chart)
            else ();
            if showderivs && (showderivcnt || showunparsable || showchart) then begin
                printf "Derivations (Bottom-up):\n";
                list_chart chart;
                printf "\n"
            end else ();
            incr totalcnt;
            if Chart.is_success chart then incr succcnt
            else ())
        chartdb;
    if showchart || showunparsable || showderivcnt then
        printf "\n"
    else ()

let main =
    Utils.print_version ();
    let fnames = ref [] 
    and showchart = ref true
    and showderivs = ref false
    and showunparsable = ref false 
    and showderivcnt = ref false in
    let args = [
        ("--nocontent",
            Arg.Clear showchart,
            "  Do not display chart contents");
        ("--derivs",
            Arg.Set showderivs,
            "  Also display all derivations");
        ("--unparsable",
            Arg.Set showunparsable,
            "  Show only unparsable charts");
        ("--derivcnt",
            Arg.Set showderivcnt,
            "  Show only derivation numbers") ]
    and fn_anon = (fun f -> fnames := !fnames @ [f])
    and usagemsg = 
        "List all charts in the specified chart databases.\n\n"
        ^ (sprintf "Usage: %s chartdb1 chartdb2 ..." Sys.argv.(0)) in
    Arg.parse args fn_anon usagemsg;
    List.iter (showchartdb !showchart !showderivs !showunparsable !showderivcnt) !fnames;
    let perc = 100.0 *. (float_of_int !succcnt /. float_of_int !totalcnt) in
    printf "Summary: success %d / total %d (%6.2f%%)\n" !succcnt !totalcnt perc
