(***********************************************************
*                                                          *
*      FUNGI - Fast UNsupervised Grammar Induction         *
*                                                          *
*   Originator: Prachya Boonkwan [1], [2]                  *
*               [1] School of Informatics,                 *
*                   University of Edinburgh, UK            *
*               [2] National Electronics and Computer      *
*                   Technology Center (NECTEC), Thailand   *
*                                                          *
*   Copyright (C) 2011 Prachya Boonkwan                    *
*   <p.boonkwan@sms.ed.ac.uk, kaamanita@hotmail.com>       *
*                                                          *
*   This library is free software; you can redistribute    *
*   it and/or modify it under the terms of the GNU         *
*   Lesser General Public License as published by the      *
*   Free Software Foundation; either version 2.1 of the    *
*   License, or (at your option) any later version.        *
*                                                          *
*   This library is distributed in the hope that it will   *
*   be useful, but WITHOUT ANY WARRANTY; without even      *
*   the implied warranty of MERCHANTABILITY or FITNESS     *
*   FOR A PARTICULAR PURPOSE.  See the GNU Lesser          *
*   General Public License for more details.               *
*                                                          *
*   You should have received a copy of the GNU Lesser      *
*   General Public License along with this library; if     *
*   not, write to the Free Software Foundation, Inc.,      *
*   59 Temple Place, Suite 330, Boston, MA  02111-1307     *
*   USA.                                                   *
*                                                          *
***********************************************************)

open Dep
open Conlldep
open Printf

let showdepdb fname =
    let id = ref 0 in
    let depstrs = ConllDep.from_file fname in
    printf "\nDependency database: %s\n\n" fname;
    List.iter
        (fun depstr ->
            incr id;
            printf "\n# Dependency structure %d\n" !id;
            printf "%s\n" (DepStruct.to_string depstr))
        depstrs

let showdepdb_conll fname =
    let id = ref 0 in
    let content = ConllDep.compile true fname in
    printf "\nDependency database: %s\n\n" fname;
    List.iter
        (fun entries ->
            incr id;
            printf "\n# Dependency structure %d\n" !id;
            printf "%s\n" (ConllDep.entries_to_conll entries))
        content

let main =
    Utils.print_version ();
    let fnames = ref [] in
    let args = []
    and fn_anon = (fun f -> fnames := !fnames @ [f])
    and usagemsg =
        "Display all dependency structures read from the specified CoNLL2006-formatted databases.\n\n"
        ^ (sprintf "Usage: %s depdb1 depdb2 ..." Sys.argv.(0)) in
    Arg.parse args fn_anon usagemsg;
    List.iter showdepdb_conll !fnames
