(***********************************************************
*                                                          *
*      FUNGI - Fast UNsupervised Grammar Induction         *
*                                                          *
*   Originator: Prachya Boonkwan [1], [2]                  *
*               [1] School of Informatics,                 *
*                   University of Edinburgh, UK            *
*               [2] National Electronics and Computer      *
*                   Technology Center (NECTEC), Thailand   *
*                                                          *
*   Copyright (C) 2011 Prachya Boonkwan                    *
*   <p.boonkwan@sms.ed.ac.uk, kaamanita@hotmail.com>       *
*                                                          *
*   This library is free software; you can redistribute    *
*   it and/or modify it under the terms of the GNU         *
*   Lesser General Public License as published by the      *
*   Free Software Foundation; either version 2.1 of the    *
*   License, or (at your option) any later version.        *
*                                                          *
*   This library is distributed in the hope that it will   *
*   be useful, but WITHOUT ANY WARRANTY; without even      *
*   the implied warranty of MERCHANTABILITY or FITNESS     *
*   FOR A PARTICULAR PURPOSE.  See the GNU Lesser          *
*   General Public License for more details.               *
*                                                          *
*   You should have received a copy of the GNU Lesser      *
*   General Public License along with this library; if     *
*   not, write to the Free Software Foundation, Inc.,      *
*   59 Temple Place, Suite 330, Boston, MA  02111-1307     *
*   USA.                                                   *
*                                                          *
***********************************************************)

open Printf

let showhmm epsilon fname =
    printf "\nHidden Markov Model: %s\n\n" fname;
    let fwdbwd = Hmm.from_file fname epsilon in
    Hmm.print fwdbwd;
    printf "\n"

let main =
    Utils.print_version ();
    let fnames = ref []
    and epsilon = ref 1e-25 in
    let args = [
        ("--epsilon",
            Arg.Set_float epsilon,
            "[float]   Set the value of epsilon (a very small value) (default = 1e-25)")]
    and fn_anon = (fun f -> fnames := !fnames @ [f])
    and usagemsg = 
        "Display the specified hidden Markov models.\n\n"
        ^ (sprintf "Usage: %s hmm1 hmm2 ..." Sys.argv.(0)) in
    Arg.parse args fn_anon usagemsg;
    List.iter (showhmm !epsilon) !fnames
