(***********************************************************
*                                                          *
*      FUNGI - Fast UNsupervised Grammar Induction         *
*                                                          *
*   Originator: Prachya Boonkwan [1], [2]                  *
*               [1] School of Informatics,                 *
*                   University of Edinburgh, UK            *
*               [2] National Electronics and Computer      *
*                   Technology Center (NECTEC), Thailand   *
*                                                          *
*   Copyright (C) 2011 Prachya Boonkwan                    *
*   <p.boonkwan@sms.ed.ac.uk, kaamanita@hotmail.com>       *
*                                                          *
*   This library is free software; you can redistribute    *
*   it and/or modify it under the terms of the GNU         *
*   Lesser General Public License as published by the      *
*   Free Software Foundation; either version 2.1 of the    *
*   License, or (at your option) any later version.        *
*                                                          *
*   This library is distributed in the hope that it will   *
*   be useful, but WITHOUT ANY WARRANTY; without even      *
*   the implied warranty of MERCHANTABILITY or FITNESS     *
*   FOR A PARTICULAR PURPOSE.  See the GNU Lesser          *
*   General Public License for more details.               *
*                                                          *
*   You should have received a copy of the GNU Lesser      *
*   General Public License along with this library; if     *
*   not, write to the Free Software Foundation, Inc.,      *
*   59 Temple Place, Suite 330, Boston, MA  02111-1307     *
*   USA.                                                   *
*                                                          *
***********************************************************)

open Dep
open Hdb
open Tree
open Utils
open Printf

module MarshTree = struct

    type tree_t = Tree.t

    type depstruct_t = DepStruct.t

    type t = (tree_t * float * depstruct_t)  option

    type key_t = int

    type value_t = t

    let create tree prob depstr : t =
        Some (tree, prob, depstr)

    let equal = Pervasives.(=)

    let hash = Hashtbl.hash

    let get_tree (mtree : t) =
        match mtree with
            Some (tree, _, _) -> tree
          | None -> failwith "get_tree"

    let get_prob (mtree : t) =
        match mtree with
            Some (_, prob, _) -> prob
          | None -> failwith "get_prob"

    let get_depstr (mtree : t) =
        match mtree with
            Some (_, _, depstr) -> depstr
          | None -> failwith "get_depstr"

    let to_string (mtree : t) =
        match mtree with
            Some (tree, prob, depstr) ->
                sprintf "%s\nProbability = %e\n<Dependency>\n%s"
                    (Tree.to_string tree) prob (DepStruct.to_string depstr)
          | None -> "None"

end

module TreeDb = SeqHDB (MarshTree)

module TreeDbWrapper = SeqHDBWrapper (MarshTree)
