// fusion_determin.hxx
//
// Mical, a regular grammar inference library
// Copyright (C) 2003 Maxime Rey
//
// This library is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public
// License as published by the Free Software Foundation; either
// version 2.1 of the License, or any later version.
//
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// Lesser General Public License for more details.

// You should have received a copy of the GNU Lesser General Public
// License along with this library; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

#ifndef MICAL_FUSION_DETERMIN_HXX_
#define MICAL_FUSION_DETERMIN_HXX_

#include <mical/algorithms/fusion_determin.hh>

#include <mical/internal/fusion.hh>

#include <mical/framework/viewer/view_lattice_elt.hh>

namespace mical {

  using namespace vcsn;

  template<class A, class T> 
  struct FusionDetermin
  {
    typedef FusionDetermin<A, T>    self_t;
    
    typedef InfoDepth<A, T, self_t>       info_t;

    typedef vector<hstate_t>              delta_state_t;

    typedef Element<A, T>                 automaton_t;
    
    
    FusionDetermin()
    {}

    void operator ()(Element<A, T>& a, 
		     hstate_t state)
    {
      depth_search(a, state, *this, security::safe_t(1));
    }

    void prefix(info_t& info)
    {
      bool flag;

      do 
	{
	  flag = false;
	  for (typename automaton_t::monoid_t::alphabet_t::iterator 
		 i = info.auto_.series().monoid().alphabet().begin();
	       i != info.auto_.series().monoid().alphabet().end(); i++)
	    {
	      set<hedge_t> delta_edge;
 	      info.auto_.letter_deltac(delta_edge, info.state_, *i, delta_kind::edges());
	  
	      hstate_t tmp;
	      if (delta_edge.size())
		tmp = info.auto_.aim_of(*delta_edge.begin());
	      if (delta_edge.size() > 1)
		{
		  for (typename set<hedge_t>::iterator i = ++delta_edge.begin();
		       i != delta_edge.end(); i++)
		    if (*i < tmp)
		      fusion(info.auto_, tmp, info.auto_.aim_of(*i));
		    else
		      fusion(info.auto_, info.auto_.aim_of(*i), tmp);
		  flag = true;
		}
	    }
	} while (flag);
    }

    void infix(info_t& info)
    {}
    
    void postfix(info_t& info)
    {}
    
    template<typename Container_>
    void 
    delta(info_t& info, Container_& res, hstate_t from)
    {
      info.auto_.deltac(res, from, delta_kind::states());
    }
    
  };
  

  template<class A, class T, class Partition_> 
  struct FusionDeterminView
  {};

  template<class T, class Partition_> 
  struct FusionDeterminView<ViewLattice, T, Partition_>
  {
    typedef FusionDeterminView<ViewLattice, T, Partition_>    self_t;
    
    typedef InfoDepth<ViewLattice, T, self_t>             info_t;
    
    typedef vector<hstate_t>                              delta_state_t;

    typedef Element<ViewLattice, T>                       automaton_t;

    typedef Partition<Partition_>                         partition_t;
    
        
    FusionDeterminView()
    {}

    void operator ()(Element<ViewLattice, T>& a, 
		     hstate_t state, 
		     partition_t& partition)
    {
      partition_ = &partition;      

      depth_search(a, state, *this, security::safe_t(1));

      a.update();
    }

    void prefix(info_t& info)
    {
      bool flag;
      do 
	{
	  flag = false;
	  for (typename automaton_t::alphabet_t::iterator 
		 i = info.auto_.alphabet().begin();
	       i != info.auto_.alphabet().end(); i++)
	    {
	      set<hedge_t> delta_edge;
	      info.auto_.deltac(delta_edge, info.state_, *i, delta_kind::edges());

	      hstate_t tmp;
	      if (delta_edge.size())
		tmp = info.auto_.aim_of(*delta_edge.begin());
	      if (delta_edge.size() > 1)
		{
		  for (typename set<hedge_t>::iterator i = ++delta_edge.begin();
		       i != delta_edge.end(); i++)
		    {
		      partition_->weld(tmp, info.auto_.aim_of(*i));
		      info.auto_.update();
		    }
		  flag = true;
		}
	    }
	} while (flag);
    }

    void infix(info_t& info)
    {}

    void postfix(info_t& info)
    {}
    
    template<typename Container_>
    void 
    delta(info_t& info, Container_& res, hstate_t from)
    {
      info.auto_.deltac(res, from, delta_kind::states());
    }
    
  private:

    partition_t* partition_;
    
  };


  template <class A, class T, class Partition_>
  void 
  fusion_determin(Element<A, T>& automaton,
		  hstate_t state,
		  Partition<Partition_>& partition)
  {
    FusionDeterminView<A, T, Partition_> fusiondetermin;

    fusiondetermin(automaton, state, partition);
  }

  template <class A, class T>
  void 
  fusion_determin(Element<A, T>& automaton,
		  hstate_t state)	
  {
    FusionDetermin<A, T> fusiondetermin;

    fusiondetermin(automaton, state);
  }



  
} // mical

#endif // MICAL_FUSION_DETERMIN_HXX_
