// is_kdeterministic.hxx
//
// Mical, a regular grammar inference library
// Copyright (C) 2003 Maxime Rey
//
// This library is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public
// License as published by the Free Software Foundation; either
// version 2.1 of the License, or any later version.
//
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// Lesser General Public License for more details.

// You should have received a copy of the GNU Lesser General Public
// License along with this library; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

#ifndef MICAL_IS_KDETERMINISTIC_HXX_
#define MICAL_IS_KDETERMINISTIC_HXX_

#include <vaucanson/fundamental/element.hh>

#include <algorithms/is_kdeterministic.hh>

#include <set>
#include <utility>

using namespace vcsn;
using std::pair;
using std::set;

namespace mical {

  // deterministic with anticipation k
  template <class A, class T>
  bool is_kdeterministic(const Element<A, T>& a, unsigned k)
  {
    typedef Element<A, T> automaton_t;

    for (typename automaton_t::state_iterator i = a.states().begin();
	 i != a.states().end(); i++)
      {
	bool res = true;
	typename Element<A, T>::monoid_elt_t str;
	set<pair<typename Element<A, T>::monoid_elt_t, hstate_t> > acu;
	set<hstate_t> marked;

	kanticip(a, *i, 0, k, str, acu, marked, res);
	if (!res)
	  return false;
      }
    
    return true;
  }
  
  template <typename A, typename T>
  void kanticip(const Element<A, T>& a, 
		hstate_t current, 
		unsigned actual,
		unsigned k,
		typename Element<A, T>::monoid_elt_t& str,
		set<pair<typename Element<A, T>::monoid_elt_t, hstate_t> >& acu,
		set<hstate_t>& marked,
		bool& res)
  {
    typedef typename Element<A, T>::monoid_elt_t Word;
    typedef set<pair<Word, hstate_t> > word_state_t;

    if (actual > k)
      {
	if (!acu.size())
	  acu.insert(pair<Word, hstate_t>(str, current));
	else
	  {
	  for (typename word_state_t::iterator i = acu.begin();
	       i != acu.end(); i++)
	    {
	      if ((i->first == str) && (i->second != current))
		res = false;
	    }
	  if (!res)
	    return;
	  else
	    acu.insert(pair<Word, hstate_t>(str, current));
	  }
      }
    if (find(marked.begin(), marked.end(), current) != marked.end())
      return;

    marked.insert(current);
    
    set<hedge_t> delta_edge;
    
    a.deltac(delta_edge, current, delta_kind::edges());
	
    for (typename set<hedge_t>::iterator i = delta_edge.begin();
	 i != delta_edge.end(); i++)
      {
	  Word svg = str;
	  str = str * a.label_of(*i).begin()->first;
	  
	  kanticip(a, a.aim_of(*i), actual + 1, k, str, acu, marked, res);
	      
	  if (res == false)
	    return;
	  
	  str = svg;
      }
  }


} // mical


#endif // MICAL_IS_KDETERMINISTIC_HXX_
