// framework.hh
//
// Mical, a regular grammar inference library
// Copyright (C) 2003 Maxime Rey
//
// This library is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public
// License as published by the Free Software Foundation; either
// version 2.1 of the License, or any later version.
//
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// Lesser General Public License for more details.

// You should have received a copy of the GNU Lesser General Public
// License along with this library; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

#ifndef MICAL_FRAMEWORK_HH_
#define MICAL_FRAMEWORK_HH_

#include <vaucanson/automata/concept/automata.hh>
#include <vaucanson/algebra/concrete/predefs.hh>

#include <mical/framework/framework_kind.hh>

#include <mical/framework/partitions/partitions.hh>
#include <mical/samples/kind/summary.hh>

#include <mical/framework/viewer/view_lattice_elt.hh>

namespace mical {

  using namespace vcsn;
  using namespace vcsn::algebra;
  using namespace vcsn::algebra::char_letter;


  template <typename SearchKind_, typename InferenceKind_> 
  struct LatticeKind
  {
    typedef SearchKind_       search_kind_t;
    typedef InferenceKind_    inference_kind_t;
  };

  
  template <typename Automaton_, 
	    typename ContainerPartition_,
	    typename ContainerSample_,
	    typename InferenceKind_>
  struct framework_traits
  {
    typedef Automaton_                                            automaton_t;
    
    typedef Partition<partition_traits<ContainerPartition_> >     partition_t;
    
    typedef Presentation<InferenceKind_, ContainerSample_, Word>  sample_t;

    typedef Element<ViewLattice, ImplemViewLattice<automaton_t, partition_t> >
    view_t;
  };
  
  template <typename SearchKind_, 
	    typename InferenceKind_,
	    typename Implem_>
  struct FrameWork
  {
    typedef SearchKind_                              search_kind_t;
    typedef InferenceKind_                           inference_kind_t;
    
    typedef typename Implem_::automaton_t            automaton_t;
    typedef typename Implem_::partition_t            partition_t;
    typedef typename partition_t::block_t            block_t;
    typedef typename Implem_::view_t                 view_t;
    typedef typename Implem_::sample_t               sample_t;
    typedef typename automaton_t::monoid_elt_t       word_t;
    
    typedef typename view_t::state_iterator          state_iterator;
    typedef typename view_t::edge_iterator           edge_iterator;
    
    FrameWork();
    FrameWork(const FrameWork& other);
    FrameWork(char* name);
    FrameWork(const sample_t& sample);
    FrameWork(const automaton_t& automaton, const sample_t& sample);

    void init();    


    void fusion(hstate_t lhs, hstate_t rhs);
    
    void fusion(state_iterator& lhs, state_iterator& rhs);

    void fission(hstate_t lhs, hstate_t rhs);
    
    void fission(state_iterator& lhs, state_iterator& rhs);

    void fission(hstate_t lhs, const block_t& rhs);



    bool is_universal() const;
    
    bool is_null() const;
    
    

    bool is_compatible() const;


    bool is_lattice_elt() const;

    bool is_lattice_elt(const automaton_t& automaton) const;
    
    bool is_borderset_elt();
    


    void determin(hstate_t state);

    template <class Any>
    void 
    apply(Any func);

    template <class Any>
    void 
    next_step(Any func);
    

    view_t* view() const;

    // to disapear (history will remplace it)
    partition_t& partition();

    automaton_t generate() const;

    
  private:
    
    automaton_t auto_;
    partition_t partition_;

    view_t*     view_;

    sample_t    sample_;
  };

} // mical

#include <mical/framework/framework.hxx>

#endif // MICAL_FRAMEWORK_HH_
