// partition_set.hh
//
// Mical, a regular grammar inference library
// Copyright (C) 2003 Maxime Rey
//
// This library is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public
// License as published by the Free Software Foundation; either
// version 2.1 of the License, or any later version.
//
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// Lesser General Public License for more details.

// You should have received a copy of the GNU Lesser General Public
// License along with this library; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

#ifndef MICAL_PARTITION_SET_HH_
# define MICAL_PARTITION_SET_HH_

#include <set>

#include <vaucanson/automata/concept/automata.hh>

namespace mical {

  using namespace vcsn;
  using std::set;

  typedef set<set<hstate_t> >                    partition_t;
  typedef set<hstate_t>                          block_t;
  typedef hstate_t                               elt_t;

  typedef partition_t::iterator         block_iterator;
  typedef block_t::iterator             elt_iterator;


  void
  proxy_implose(partition_t& partition, 
		const block_iterator& i);

  void 
  proxy_extract(partition_t& partition,
		const block_iterator& block, 
		elt_t elt);
  
  void 
  proxy_extract(partition_t& partition,
		const block_iterator& block, 
		const block_t& target);
   
  void 
  proxy_weld(partition_t& partition,
	     const block_iterator& block, 
	     const block_iterator& target);

  void 
  proxy_insert(partition_t& partition,
	       const block_t& block);

  void 
  proxy_insert(partition_t& partition,
	       elt_t elt);

  void 
  proxy_insert(partition_t& partition,
	       const block_iterator& block, 
	       elt_t elt);

  void 
  proxy_erase(partition_t& partition,
	      const block_iterator& block);

  void 
  proxy_erase(partition_t& partition,
	      const block_iterator& block, 
	      elt_t elt);

} // mical

#include <mical/framework/partitions/concrete/partition_set.hxx>

#endif // MICAL_PARTITION_SET_HH_
