// is_compatible.hxx
//
// Mical, a regular grammar inference library
// Copyright (C) 2003 Maxime Rey
//
// This library is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public
// License as published by the Free Software Foundation; either
// version 2.1 of the License, or any later version.
//
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// Lesser General Public License for more details.

// You should have received a copy of the GNU Lesser General Public
// License along with this library; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA


#ifndef MICAL_IS_COMPATIBLE_HXX_
#define MICAL_IS_COMPATIBLE_HXX_

#include <mical/internal/is_compatible.hh>

#include <mical/primitives/compute_one.hh>

namespace mical {

  using namespace vcsn;
  
  template<typename A, typename T, typename Container_>
  bool
  is_compatible(const Element<A, T>& a, 
		const FixedSample<Container_,
		typename Element<A, T>::monoid_elt_t>& sample)
  {
    typedef FixedSample<Container_,
      typename Element<A, T>::monoid_elt_t> sample_t;

    for (typename sample_t::sample_iterator i = sample.begin();
	 i != sample.end(); i++)
      if (compute_one(a, *i, compute_kind::boolean()))
	return false;
      
    return true;
  }


  template<typename A, typename T, typename Kind_, typename Container_>
  bool
  is_compatible(const Element<A, T>& a, 
		const Presentation<Kind_, Container_,
		typename Element<A, T>::monoid_elt_t>& sample)
  {
    typedef Presentation<Kind_, Container_,
      typename Element<A, T>::monoid_elt_t> sample_t;
    
    for (typename sample_t::sample_iterator i = sample.n_begin();
	 i != sample.n_end(); i++)
      if (compute_one(a, *i, compute_kind::boolean()))
	return false;
    
    return true;
  }


} // mical

#endif // MICAL_IS_COMPATIBLE_HXX_
