// k_window.hh
//
// Mical, a regular grammar inference library
// Copyright (C) 2003 Maxime Rey
//
// This library is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public
// License as published by the Free Software Foundation; either
// version 2.1 of the License, or any later version.
//
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// Lesser General Public License for more details.

// You should have received a copy of the GNU Lesser General Public
// License along with this library; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

#ifndef MICAL_K_WINDOW_HH_
#define MICAL_K_WINDOW_HH_

#include <vaucanson/algebra/concrete/predefs.hh>
#include <queue>

using std::queue;

namespace mical {

  template <class Letter>
  struct KWindow
  {
    KWindow(unsigned k); 
    
    template <template<class> class Value1, template<class> class Value2>
    KWindow(unsigned k,
	    Element<
	    FreeMonoid<Element<AlphabetSet<Letter>, Value1<Letter> > >, 
	    Value2<Letter> > str); 

    template <template<class> class Value1, template<class> class Value2>
    void insert(Element<
		FreeMonoid<Element<AlphabetSet<Letter>, Value1<Letter> > >, 
		Value2<Letter> > str);

    void insert(Letter l);

    unsigned size();

    void reduce(unsigned size);

    void clear();

    template <template<class> class Value1, template<class> class Value2>
    Element<
      FreeMonoid<Element<AlphabetSet<Letter>, Value1<Letter> > >, 
      Value2<Letter> 
    > get_window(Element<
		 FreeMonoid<Element<AlphabetSet<Letter>, Value1<Letter> > >, 
		 Value2<Letter> > sample);
    

  private:

    unsigned k_;
    queue<Letter> window_;

  };

} // mical

#include <mical/internal/k_window.hxx>

#endif // MICAL_K_WINDOW_HH_
