// make_pta.hxx
//
// Mical, a regular grammar inference library
// Copyright (C) 2003 Maxime Rey
//
// This library is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public
// License as published by the Free Software Foundation; either
// version 2.1 of the License, or any later version.
//
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// Lesser General Public License for more details.

// You should have received a copy of the GNU Lesser General Public
// License along with this library; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

#ifndef MICAL_MAKE_PTA_HXX_
#define MICAL_MAKE_PTA_HXX_

#include <set>

#include <mical/internal/make_pta.hh>

namespace mical {

  using namespace vcsn;
  using namespace vcsn::algebra;
  using std::set;

  template<typename A, typename T, typename Container_>
  void
  make_pta(Element<A, T>& a, 
	   const FixedSample<Container_,
	   typename Element<A, T>::monoid_elt_t>& sample)
  {
    typedef typename Element<A, T>::monoid_elt_t  word_t;
    typedef   
      FixedSample<Container_, typename Element<A, T>::monoid_elt_t>
      sample_t;
    typedef std::set<hedge_t>    delta_ret_t;

    hstate_t init = a.add_state();
    a.set_initial(init);

    for (typename sample_t::sample_iterator i = sample.begin(); 
	 i != sample.end(); ++i)
      {
	hstate_t from = init;
	hstate_t to;
	delta_ret_t res;
	bool flag;
	bool pas = true;
    
	for (typename word_t::const_iterator j = i->begin(); j != i->end(); ++j)
	  {
	    hstate_t ret;
	    flag = true;
	    
	    if (pas)
	      {
		res.clear();
		a.deltac(res, from, delta_kind::edges());
		for (delta_ret_t::iterator k = res.begin(); k != res.end(); ++k)
		  {
		    if (*a.label_of(*k).begin()->first.begin() == *j)
		      {
			flag = false;
			ret = a.aim_of(*k);
		      }
		  }
	      }
	    
 	    if (flag)
 	      {
		pas = false;
		to = a.add_state();
		a.add_letter_edge(from, to, *j);
 	      }
	    else 
	      to = ret;
	    
	    from = to;
	  }
	a.set_final(to);
      }
  }
  

  template<typename A, typename T, typename Kind_, typename Container_>
  void
  make_pta(Element<A, T>& a, 
	   const Presentation<Kind_, Container_,
	   typename Element<A, T>::monoid_elt_t>& sample)
  {
    typedef typename Element<A, T>::monoid_elt_t  word_t;
    typedef   
      Presentation<Kind_, Container_, typename Element<A, T>::monoid_elt_t>
      sample_t;
    typedef std::set<hedge_t>    delta_ret_t;

    hstate_t init = a.add_state();
    a.set_initial(init);

    for (typename sample_t::sample_iterator i = sample.p_begin(); 
	 i != sample.p_end(); ++i)
      {
	hstate_t from = init;
	hstate_t to;
	delta_ret_t res;
	bool flag;
	bool pas = true;
    
	for (typename word_t::const_iterator j = i->begin(); j != i->end(); ++j)
	  {
	    hstate_t ret;
	    flag = true;
	    
	    if (pas)
	      {
		res.clear();
		a.deltac(res, from, delta_kind::edges());
		for (delta_ret_t::iterator k = res.begin(); k != res.end(); ++k)
		  {
		    if (*a.label_of(*k).begin()->first.begin() == *j)
		      {
			flag = false;
			ret = a.aim_of(*k);
		      }
		  }
	      }
	    
 	    if (flag)
 	      {
		pas = false;
		to = a.add_state();
		a.add_letter_edge(from, to, *j);
 	      }
	    else 
	      to = ret;
	    
	    from = to;
	  }
	a.set_final(to);
      }
  }

  template<typename A, typename T, typename Container_>
  void
  make_pta(Element<A, T>& a, 
	   const FixedSample<Container_,
	   typename Element<A, T>::monoid_elt_t>& sample,
	   map<hstate_t, unsigned>& mappy)
  {
    typedef typename Element<A, T>::monoid_elt_t  word_t;
    typedef   
      FixedSample<Container_, typename Element<A, T>::monoid_elt_t>
      sample_t;
    typedef std::set<hedge_t>    delta_ret_t;
    
    hstate_t init = a.add_state();
    a.set_initial(init);

    mappy[init] = 0;

    for (typename sample_t::sample_iterator i = sample.begin(); 
	 i != sample.end(); ++i)
      {
	hstate_t from = init;
	hstate_t to;
	delta_ret_t res;
	bool flag;
	bool pas = true;
	
	mappy[init]++;

	for (typename word_t::const_iterator j = i->begin(); j != i->end(); ++j)
	  {
	    hstate_t ret;
	    flag = true;
	    
	    if (pas)
	      {
		res.clear();
		a.deltac(res, from, delta_kind::edges());
		for (delta_ret_t::iterator k = res.begin(); k != res.end(); ++k)
		  {
		    if (*a.label_of(*k).begin()->first.begin() == *j)
		      {
			flag = false;
			ret = a.aim_of(*k);
		      }
		  }
	      }
	    
 	    if (flag)
 	      {
		pas = false;
		to = a.add_state();
		mappy[to] = 1;
		a.add_letter_edge(from, to, *j);
 	      }
	    else 
	      {
		to = ret;
		mappy[to]++;
	      }
	    
	    from = to;
	  }
	a.set_final(to);
      }
  }
  

  template<typename A, typename T, typename Kind_, typename Container_>
  void
  make_pta(Element<A, T>& a, 
	   const Presentation<Kind_, Container_,
	   typename Element<A, T>::monoid_elt_t>& sample,
	   map<hstate_t, unsigned>& mappy)
  {
    typedef typename Element<A, T>::monoid_elt_t  word_t;
    typedef   
      Presentation<Kind_, Container_, typename Element<A, T>::monoid_elt_t>
      sample_t;
    typedef std::set<hedge_t>    delta_ret_t;
    
    hstate_t init = a.add_state();
    a.set_initial(init);

    mappy[init] = 0;

    for (typename sample_t::sample_iterator i = sample.p_begin(); 
	 i != sample.p_end(); ++i)
      {
	hstate_t from = init;
	hstate_t to;
	delta_ret_t res;
	bool flag;
	bool pas = true;
	
	mappy[init]++;

	for (typename word_t::const_iterator j = i->begin(); j != i->end(); ++j)
	  {
	    hstate_t ret;
	    flag = true;
	    
	    if (pas)
	      {
		res.clear();
		a.deltac(res, from, delta_kind::edges());
		for (delta_ret_t::iterator k = res.begin(); k != res.end(); ++k)
		  {
		    if (*a.label_of(*k).begin()->first.begin() == *j)
		      {
			flag = false;
			ret = a.aim_of(*k);
		      }
		  }
	      }
	    
 	    if (flag)
 	      {
		pas = false;
		to = a.add_state();
		mappy[to] = 1;
		a.add_letter_edge(from, to, *j);
 	      }
	    else 
	      {
		to = ret;
		mappy[to]++;
	      }
	    
	    from = to;
	  }
	a.set_final(to);
      }
  }

} // mical

#endif
