// swap_states.hxx
//
// Mical, a regular grammar inference library
// Copyright (C) 2003 Maxime Rey
//
// This library is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public
// License as published by the Free Software Foundation; either
// version 2.1 of the License, or any later version.
//
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// Lesser General Public License for more details.

// You should have received a copy of the GNU Lesser General Public
// License along with this library; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

#ifndef MICAL_SWAP_STATES_HXX_
# define MICAL_SWAP_STATES_HXX_

#include <mical/internal/swap_states.hh>
#include <mical/tools/predefs.hh>
#include <set>
#include <algorithm>

using namespace vcsn;
using std::set;
using std::set_difference;

using namespace std;

namespace mical {

  template <class A, class T>
  void swap_states(Element<A, T>& a, hstate_t lhs, hstate_t rhs)
  {
    if (lhs == rhs)
      return;

    set<hedge_t> delta_edge1;

    // bufferization of edges of the state rhs 
    a.deltac(delta_edge1, rhs, delta_kind::edges());
    a.rdeltac(delta_edge1, rhs, delta_kind::edges());
	
    set<hedge_t> tmp, delta_edge2;

    a.deltac(tmp, lhs, delta_kind::edges());
    a.rdeltac(tmp, lhs, delta_kind::edges());

    set_difference(tmp.begin(), tmp.end(),
		   delta_edge1.begin(), delta_edge1.end(),
		   inserter(delta_edge2, delta_edge2.begin()));

    // add edges of the lhs state to the state rhs
    for (set<hedge_t>::iterator i = delta_edge2.begin();
	 i != delta_edge2.end(); i++)
      if (a.aim_of(*i) == lhs)
	if (a.origin_of(*i) == rhs)
	  a.add_edge(lhs, rhs, a.label_of(*i));
	else
	  a.add_edge(a.origin_of(*i), rhs, a.label_of(*i));
      else
	if (a.aim_of(*i) == rhs)
	  a.add_edge(rhs, lhs, a.label_of(*i));
	else
	  a.add_edge(rhs, a.aim_of(*i), a.label_of(*i));

    // add edges of the state rhs to the lhs state
    for (set<hedge_t>::iterator i = delta_edge1.begin();
	 i != delta_edge1.end(); i++)
	if (a.aim_of(*i) == rhs)
	  if (a.origin_of(*i) == lhs)
	    a.add_edge(rhs, lhs, a.label_of(*i));
	  else
	    a.add_edge(a.origin_of(*i), lhs, a.label_of(*i));
	else
	  if (a.aim_of(*i) == lhs)
	    a.add_edge(lhs, rhs, a.label_of(*i));
	  else
	    a.add_edge(lhs, a.aim_of(*i), a.label_of(*i));

    // deletion of edges of the lhs state 
    for (set<hedge_t>::iterator i = delta_edge2.begin();
	 i != delta_edge2.end(); i++)
      del_edge(a, *i);
    
    for (set<hedge_t>::iterator i = delta_edge1.begin();
	 i != delta_edge1.end(); i++)
      del_edge(a, *i);

    bool i_lhs = false, i_rhs = false, f_lhs = false, f_rhs = false;

    if (a.is_initial(lhs)) i_lhs = true;
    if (a.is_final(lhs)) f_lhs = true;
    if (a.is_initial(rhs)) i_rhs = true;
    if (a.is_final(rhs)) f_rhs = true;

    if (i_lhs)
      {
	a.unset_initial(lhs);
	a.set_initial(rhs);
      }
    
    if (f_lhs)
      {
	a.unset_final(lhs);
	a.set_final(rhs);
      }
    
    if (i_rhs)
      {
	a.unset_initial(rhs);
	a.set_initial(lhs);
      }
    
    if (f_rhs)
      {
	a.unset_final(rhs);
	a.set_final(lhs);
      }

  }


} // mical

#endif /* !SWAP_STATES_HXX_ */
