// compute.hxx
//
// Mical, a regular grammar inference library
// Copyright (C) 2003 Maxime Rey
//
// This library is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public
// License as published by the Free Software Foundation; either
// version 2.1 of the License, or any later version.
//
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// Lesser General Public License for more details.

// You should have received a copy of the GNU Lesser General Public
// License along with this library; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

#ifndef MICAL_COMPUTE_HXX_
#define MICAL_COMPUTE_HXX_

#include <mical/walkers/depth_search.hh>
#include <mical/primitives/compute.hh>

#include <algorithm>

namespace mical {

  using std::copy;
  using std::find;

  template<class A, class T, class Word_> 
  Compute_n<A, T, Word_>::Compute_n()
  {}

  template<class A, class T, class Word_> 
  bool 
  Compute_n<A, T, Word_>::operator ()(Element<A, T> a, 
				      Word_ word, 
				      unsigned n, 
				      compute_kind::boolean)
  {
    set<hedge_t> tmp;

    this->operator()(a, word, n, tmp);
    
    return ((tmp.size() != 0)
	    || ((tmp.size() == 0) && 
		!word.value().size() && 
		(a.is_final(*a.initial().begin()))));
  }

  template<class A, class T, class Word_> 
  int 
  Compute_n<A, T, Word_>::operator ()(Element<A, T> a, 
				      Word_ word, 
				      unsigned n, 
				      compute_kind::state)
  {
    str = word;

    p = str.begin();
    
    path_edges.clear();
    count = 1;
    max_n = n;

    for (typename Element<A, T>::edge_iterator i = a.edges().begin(); 
	 i != a.edges().end(); i++)
      path_edges[*i] = 0;

    final_state = -1;
    
    depth_search(a, *this, security::responsable_t());

    return final_state;
  }

  template<class A, class T, class Word_> 
  template<template<class> class Container_>
  void
  Compute_n<A, T, Word_>::operator ()(Element<A, T> a, 
				      Word_ word, 
				      unsigned n, 
				      Container_<hstate_t>& res)
  {
    set<hedge_t> tmp1;

    this->operator()(a, word, n, tmp1);

    set<hstate_t> tmp2;
    for (typename set<hedge_t>::iterator i = tmp1.begin();
	 i != tmp1.end(); i++)
      {
	tmp2.insert(a.origin_of(*i));
	tmp2.insert(a.aim_of(*i));
      }

    copy(tmp2.begin(), tmp2.end(),
	 inserter(res, res.begin()));
  }

  template<class A, class T, class Word_> 
  template<template<class> class Container_>
  void
  Compute_n<A, T, Word_>::operator ()(Element<A, T> a, 
				      Word_ word, 
				      unsigned n, 
				      Container_<hedge_t>& res)
  {
    str = word;

    if (str.value().size() == 0)
      return;

    p = str.begin();
    
    path_edges.clear();
    count = 1;
    max_n = n;

    for (typename Element<A, T>::edge_iterator i = a.edges().begin(); 
	 i != a.edges().end(); i++)
      path_edges[*i] = 0;
    
    depth_search(a, *this, security::responsable_t());

    set<hedge_t> tmp;
    for (typename path_edges_t::iterator i = path_edges.begin();
	 i != path_edges.end(); i++)
      if (i->second > 0)
	tmp.insert(i->first);

    copy(tmp.begin(), tmp.end(),
	 inserter(res, res.begin()));
  }
  
  template<class A, class T, class Word_> 
  void 
  Compute_n<A, T, Word_>::prefix(InfoDepth<A, T, self_t>& info)
  {
    if (!info.first_)
      {
	path_edges[info.edge_] += 1;
	++p;
      }
      
    if ((find(str.begin(), str.end(), *p) == str.end()) 
	&& info.auto_.is_final(info.state_))
      {
	if (count >= max_n)
	  {
	    info.stop_ = true;
	    final_state = info.state_;
	  }
	else
	  count++;
      }

  }
  
  template<class A, class T, class Word_> 
  void 
  Compute_n<A, T, Word_>::infix(InfoDepth<A, T, self_t>& info)
  {
  }

  template<class A, class T, class Word_> 
  void 
  Compute_n<A, T, Word_>::postfix(InfoDepth<A, T, self_t>& info)
  {
    if (info.stop_)
      return;
    
    if (!info.first_)
      {
	path_edges[info.edge_] -= 1;
	p--;
      }
  }

  template<class A, class T, class Word_> 
  template<typename Container_>
  void 
  Compute_n<A, T, Word_>::delta(info_t& info, Container_& res, hstate_t from)
  {
    info.auto_.letter_deltac(res, from, *p, delta_kind::states());
  }


} // mical

#endif // MICAL_COMPUTE_HXX_
