// presentation.hxx
//
// Mical, a regular grammar inference library
// Copyright (C) 2003 Maxime Rey
//
// This library is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public
// License as published by the Free Software Foundation; either
// version 2.1 of the License, or any later version.
//
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// Lesser General Public License for more details.

// You should have received a copy of the GNU Lesser General Public
// License along with this library; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

#ifndef MICAL_PRESENTATION_HXX_
# define MICAL_PRESENTATION_HXX_

#include <mical/samples/kind/presentation.hh>
#include <mical/samples/kind/concrete/predefs.hh>

namespace mical {

  template <typename Kind_, typename Container_, typename Word_>
  Presentation<Kind_, Container_, Word_>::Presentation()
  {
    init();
  }
  
  template <typename Kind_, typename Container_, typename Word_>
  void                      
  Presentation<Kind_, Container_, Word_>::add_positive(const word_t& word)
  {
    PROXY2(add_word, word, positive_);
  }

  template <typename Kind_, typename Container_, typename Word_>
  void                      
  Presentation<Kind_, Container_, Word_>::add_negative(const word_t& word)
  {
    PROXY2(add_word, word, negative_);
  }

  template <typename Kind_, typename Container_, typename Word_>
  void                         
  Presentation<Kind_, Container_, Word_>::del_positive(const word_t& word)
  {
    PROXY2(del_word, word, positive_);
  }
  
  template <typename Kind_, typename Container_, typename Word_>
  void                         
  Presentation<Kind_, Container_, Word_>::del_negative(const word_t& word)
  {
    PROXY2(del_word, word, negative_);
  }

  template <typename Kind_, typename Container_, typename Word_>
  size_t                       
  Presentation<Kind_, Container_, Word_>::size_positive() const
  {
    return PROXY1(size, positive_step_);
  }

  template <typename Kind_, typename Container_, typename Word_>
  size_t                       
  Presentation<Kind_, Container_, Word_>::size_negative() const
  {
    return PROXY1(size, negative_step_);
  }

  template <typename Kind_, typename Container_, typename Word_>
  size_t                       
  Presentation<Kind_, Container_, Word_>::size() const
  {
    return size_positive() + size_negative();
  }

  template <typename Kind_, typename Container_, typename Word_>
  bool                         
  Presentation<Kind_, Container_, Word_>::empty_positive() const
  {
    return PROXY1(empty, positive_step_);
  }

  template <typename Kind_, typename Container_, typename Word_>
  bool                         
  Presentation<Kind_, Container_, Word_>::empty_negative() const
  {
    return PROXY1(empty, negative_step_);
  }

  template <typename Kind_, typename Container_, typename Word_>
  bool                         
  Presentation<Kind_, Container_, Word_>::empty() const
  {
    return empty_positive() && empty_negative();
  }
  
  template <typename Kind_, typename Container_, typename Word_>
  typename Container_::iterator               
  Presentation<Kind_, Container_, Word_>::p_begin()
  {
    return PROXY2(begin, positive_step_, kind_iterator::not_const());
  }
  
  template <typename Kind_, typename Container_, typename Word_>
  typename Container_::const_iterator               
  Presentation<Kind_, Container_, Word_>::p_begin() const
  {
    return PROXY2(begin, positive_step_, kind_iterator::not_mutable());
  }

  template <typename Kind_, typename Container_, typename Word_>
  typename Container_::reverse_iterator       
  Presentation<Kind_, Container_, Word_>::p_rbegin()
  {
    return PROXY2(rbegin, positive_step_, kind_iterator::not_const());
  }

  template <typename Kind_, typename Container_, typename Word_>
  typename Container_::iterator               
  Presentation<Kind_, Container_, Word_>::n_begin()
  {
    return PROXY2(begin, negative_step_, kind_iterator::not_const());
  }

  template <typename Kind_, typename Container_, typename Word_>
  typename Container_::const_iterator               
  Presentation<Kind_, Container_, Word_>::n_begin() const
  {
    return PROXY2(begin, negative_step_, kind_iterator::not_mutable());
  }

  template <typename Kind_, typename Container_, typename Word_>
  typename Container_::reverse_iterator       
  Presentation<Kind_, Container_, Word_>::n_rbegin()
  {
    return PROXY2(rbegin, negative_step_, kind_iterator::not_const());
  }
  

  template <typename Kind_, typename Container_, typename Word_>
  typename Container_::iterator               
  Presentation<Kind_, Container_, Word_>::p_end()
  {
    return PROXY2(end, positive_step_, kind_iterator::not_const());
  }

  template <typename Kind_, typename Container_, typename Word_>
  typename Container_::const_iterator               
  Presentation<Kind_, Container_, Word_>::p_end() const
  {
    return PROXY2(end, positive_step_, kind_iterator::not_mutable());
  }

  template <typename Kind_, typename Container_, typename Word_>
  typename Container_::reverse_iterator       
  Presentation<Kind_, Container_, Word_>::p_rend()
  {
    return PROXY2(rend, positive_step_, kind_iterator::not_const());
  }

  template <typename Kind_, typename Container_, typename Word_>
  typename Container_::iterator               
  Presentation<Kind_, Container_, Word_>::n_end()
  {
    return PROXY2(end, negative_step_, kind_iterator::not_const());
  }

  template <typename Kind_, typename Container_, typename Word_>
  typename Container_::const_iterator               
  Presentation<Kind_, Container_, Word_>::n_end() const
  {
    return PROXY2(end, negative_step_, kind_iterator::not_mutable());
  }

  template <typename Kind_, typename Container_, typename Word_>
  typename Container_::reverse_iterator       
  Presentation<Kind_, Container_, Word_>::n_rend()
  {
    return PROXY2(rend, negative_step_, kind_iterator::not_const());
  }
  

  template <typename Kind_, typename Container_, typename Word_>
  void 
  Presentation<Kind_, Container_, Word_>::clear_positive()
  {
    PROXY1(clear, positive_step_);
  }

  template <typename Kind_, typename Container_, typename Word_>
  void 
  Presentation<Kind_, Container_, Word_>::clear_negative()
  {
    PROXY1(clear, negative_step_);
  }

  template <typename Kind_, typename Container_, typename Word_>
  void 
  Presentation<Kind_, Container_, Word_>::clear()
  {
    clear_positive();
    clear_negative();
  }
  
  template <typename Kind_, typename Container_, typename Word_>
  Container_
  Presentation<Kind_, Container_, Word_>::negative() const
  {
    return negative_step_;
  }

  template <typename Kind_, typename Container_, typename Word_>
  Container_
  Presentation<Kind_, Container_, Word_>::positive() const
  {
    return positive_step_;
  }

  template <typename Kind_, typename Container_, typename Word_>
  Container_&
  Presentation<Kind_, Container_, Word_>::negative_container()
  {
    return negative_step_;
  }

  template <typename Kind_, typename Container_, typename Word_>
  Container_&
  Presentation<Kind_, Container_, Word_>::positive_container()
  {
    return positive_step_;
  }


  template <typename Kind_, typename Container_, typename Word_>
  void
  Presentation<Kind_, Container_, Word_>::init()
  {
    clear();
    PROXY5(init, kind_t(), count_pos, count_neg, negative_, negative_step_);
  }
  
  template <typename Kind_, typename Container_, typename Word_>
  void
  Presentation<Kind_, Container_, Word_>::next_step()
  {
    unsigned prev_count_pos = count_pos;
    unsigned prev_count_neg = count_neg;

    bool flag = true;

    do 
      {

	if ((count_pos != positive_.size()) || (count_neg != negative_.size()))
	  {
	    PROXY3(next_step, kind_t(), count_pos, count_neg);
	  }
	else 
	  flag = false;
	
	if (count_pos > positive_.size())
	  count_pos--;
	if (count_neg > negative_.size())
	  count_neg--;
	
	if (count_pos > prev_count_pos)
	  {
	    flag = false;

	    word_t word;
	    typename container_t::iterator i;
	    for (i = positive_.begin(); i != positive_.end(); i++)
	      if (find(positive_step_.begin(), positive_step_.end(), *i) 
		  == positive_step_.end())
		word = *i;

	    PROXY2(add_word, word, positive_step_);
	  }

	if (count_neg > prev_count_neg)
	  {
	    flag = false; 

	    word_t word;
	    typename container_t::iterator i;
	    for (i = negative_.begin(); i != negative_.end(); i++)
	      if (find(negative_step_.begin(), negative_step_.end(), *i) 
		  == negative_step_.end())
		word = *i;
	
	    PROXY2(add_word, word, negative_step_);
	  }
      } while (flag);
  }


  template <typename Kind_, typename Container_, typename Word_>
  Presentation<Kind_, Container_, Word_>&
  Presentation<Kind_, Container_, Word_>::
  operator=(const Presentation<Kind_, Container_, Word_>& other)
  {
    typedef typename 
      Presentation<Kind_, Container_, Word_>::sample_iterator 
      sample_iterator_t;

    this->clear();

    for (sample_iterator_t i = other.p_begin(); i != other.p_end(); i++)
      this->add_positive(*i);
    
    for (sample_iterator_t i = other.n_begin(); i != other.n_end(); i++)
      this->add_negative(*i);

    return *this;
  }

  ///////////////////////////////////////////////////////////////////////////


  template <typename Container_, typename Word_>
  Presentation<inference_kind::fixed, Container_, Word_>::Presentation()
  {}
  
  template <typename Container_, typename Word_>
  void                      
  Presentation<inference_kind::fixed, Container_, Word_>::add_positive(const word_t& word)
  {
    PROXY2(add_word, word, positive_);
  }

  template <typename Container_, typename Word_>
  void                      
  Presentation<inference_kind::fixed, Container_, Word_>::add_negative(const word_t& word)
  {
    PROXY2(add_word, word, negative_);
  }

  template <typename Container_, typename Word_>
  void                         
  Presentation<inference_kind::fixed, Container_, Word_>::del_positive(const word_t& word)
  {
    PROXY2(del_word, word, positive_);
  }
  
  template <typename Container_, typename Word_>
  void                         
  Presentation<inference_kind::fixed, Container_, Word_>::del_negative(const word_t& word)
  {
    PROXY2(del_word, word, negative_);
  }

  template <typename Container_, typename Word_>
  size_t                       
  Presentation<inference_kind::fixed, Container_, Word_>::size_positive() const
  {
    return PROXY1(size, positive_);
  }

  template <typename Container_, typename Word_>
  size_t                       
  Presentation<inference_kind::fixed, Container_, Word_>::size_negative() const
  {
    return PROXY1(size, negative_);
  }

  template <typename Container_, typename Word_>
  size_t                       
  Presentation<inference_kind::fixed, Container_, Word_>::size() const
  {
    return size_positive() + size_negative();
  }

  template <typename Container_, typename Word_>
  bool                         
  Presentation<inference_kind::fixed, Container_, Word_>::empty_positive() const
  {
    return PROXY1(empty, positive_);
  }

  template <typename Container_, typename Word_>
  bool                         
  Presentation<inference_kind::fixed, Container_, Word_>::empty_negative() const
  {
    return PROXY1(empty, negative_);
  }

  template <typename Container_, typename Word_>
  bool                         
  Presentation<inference_kind::fixed, Container_, Word_>::empty() const
  {
    return empty_positive() && empty_negative();
  }
  
  template <typename Container_, typename Word_>
  typename Container_::iterator               
  Presentation<inference_kind::fixed, Container_, Word_>::p_begin()
  {
    return PROXY2(begin, positive_, kind_iterator::not_const());
  }
  
  template <typename Container_, typename Word_>
  typename Container_::const_iterator               
  Presentation<inference_kind::fixed, Container_, Word_>::p_begin() const
  {
    return PROXY2(begin, positive_, kind_iterator::not_mutable());
  }

  template <typename Container_, typename Word_>
  typename Container_::reverse_iterator       
  Presentation<inference_kind::fixed, Container_, Word_>::p_rbegin()
  {
    return PROXY2(rbegin, positive_, kind_iterator::not_const());
  }

  template <typename Container_, typename Word_>
  typename Container_::iterator               
  Presentation<inference_kind::fixed, Container_, Word_>::n_begin()
  {
    return PROXY2(begin, negative_, kind_iterator::not_const());
  }

  template <typename Container_, typename Word_>
  typename Container_::const_iterator               
  Presentation<inference_kind::fixed, Container_, Word_>::n_begin() const
  {
    return PROXY2(begin, negative_, kind_iterator::not_mutable());
  }

  template <typename Container_, typename Word_>
  typename Container_::reverse_iterator       
  Presentation<inference_kind::fixed, Container_, Word_>::n_rbegin()
  {
    return PROXY2(rbegin, negative_, kind_iterator::not_const());
  }
  

  template <typename Container_, typename Word_>
  typename Container_::iterator               
  Presentation<inference_kind::fixed, Container_, Word_>::p_end()
  {
    return PROXY2(end, positive_, kind_iterator::not_const());
  }

  template <typename Container_, typename Word_>
  typename Container_::const_iterator               
  Presentation<inference_kind::fixed, Container_, Word_>::p_end() const
  {
    return PROXY2(end, positive_, kind_iterator::not_mutable());
  }

  template <typename Container_, typename Word_>
  typename Container_::reverse_iterator       
  Presentation<inference_kind::fixed, Container_, Word_>::p_rend()
  {
    return PROXY2(rend, positive_, kind_iterator::not_const());
  }

  template <typename Container_, typename Word_>
  typename Container_::iterator               
  Presentation<inference_kind::fixed, Container_, Word_>::n_end()
  {
    return PROXY2(end, negative_, kind_iterator::not_const());
  }

  template <typename Container_, typename Word_>
  typename Container_::const_iterator               
  Presentation<inference_kind::fixed, Container_, Word_>::n_end() const
  {
    return PROXY2(end, negative_, kind_iterator::not_mutable());
  }

  template <typename Container_, typename Word_>
  typename Container_::reverse_iterator       
  Presentation<inference_kind::fixed, Container_, Word_>::n_rend()
  {
    return PROXY2(rend, negative_, kind_iterator::not_const());
  }
  

  template <typename Container_, typename Word_>
  void 
  Presentation<inference_kind::fixed, Container_, Word_>::clear_positive()
  {
    PROXY1(clear, positive_);
  }

  template <typename Container_, typename Word_>
  void 
  Presentation<inference_kind::fixed, Container_, Word_>::clear_negative()
  {
    PROXY1(clear, negative_);
  }

  template <typename Container_, typename Word_>
  void 
  Presentation<inference_kind::fixed, Container_, Word_>::clear()
  {
    clear_positive();
    clear_negative();
  }
  
  template <typename Container_, typename Word_>
  Container_
  Presentation<inference_kind::fixed, Container_, Word_>::negative() const
  {
    return negative_;
  }

  template <typename Container_, typename Word_>
  Container_
  Presentation<inference_kind::fixed, Container_, Word_>::positive() const
  {
    return positive_;
  }

  template <typename Container_, typename Word_>
  Container_&
  Presentation<inference_kind::fixed, Container_, Word_>::negative_container()
  {
    return negative_;
  }

  template <typename Container_, typename Word_>
  Container_&
  Presentation<inference_kind::fixed, Container_, Word_>::positive_container()
  {
    return positive_;
  }


  template <typename Container_, typename Word_>
  Presentation<inference_kind::fixed, Container_, Word_>&
  Presentation<inference_kind::fixed, Container_, Word_>::
  operator=(const Presentation<inference_kind::fixed, Container_, Word_>& other)
  {
    typedef typename 
      Presentation<inference_kind::fixed, Container_, Word_>::sample_iterator 
      sample_iterator_t;

    this->clear();

    for (sample_iterator_t i = other.p_begin(); i != other.p_end(); i++)
      this->add_positive(*i);
    
    for (sample_iterator_t i = other.n_begin(); i != other.n_end(); i++)
      this->add_negative(*i);

    return *this;
  }

  ///////////////////////////////////////////////////////////////////////////
  template <typename Kind_, typename Container_, typename Word_>
  bool 
  operator==(const Presentation<Kind_, Container_, Word_>& lhs,
	     const Presentation<Kind_, Container_, Word_>& rhs)
  {
    return ((lhs.positive() == rhs.positive())
	    && (lhs.negative() == rhs.negative()));
  }

  template <typename Kind_, typename Container_, typename Word_>
  bool 
  operator!=(const Presentation<Kind_, Container_, Word_>& lhs,
	     const Presentation<Kind_, Container_, Word_>& rhs)
  {
    return !(lhs == rhs);
  }	    

} // mical

#endif // MICAL_PRESENTATION_HXX_
