// give_alphabet.hxx
//
// Mical, a regular grammar inference library
// Copyright (C) 2003 Maxime Rey
//
// This library is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public
// License as published by the Free Software Foundation; either
// version 2.1 of the License, or any later version.
//
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// Lesser General Public License for more details.

// You should have received a copy of the GNU Lesser General Public
// License along with this library; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

#ifndef MICAL_GIVE_ALPHABET_HXX_
#define MICAL_GIVE_ALPHABET_HXX_

#include <mical/samples/services/give_alphabet.hh>

#include <algorithm>
#include <set>

namespace mical {
  
  using std::set;
  using std::copy;
  using namespace vcsn;

  template <typename A, typename T, typename Container_>
  void 
  give_alphabet(const Element<A, T>& a, 
		const FixedSample<Container_,
		typename Element<A, T>::monoid_elt_t>& sample,
		typename Element<A, T>::monoid_t::alphabet_t& alphabet)
  {
    typedef 
      FixedSample<Container_, typename Element<A, T>::monoid_elt_t> 
      sample_t;
    
    set<typename Element<A, T>::monoid_t::alphabet_t::letter_t> alpha;
    
    for (typename sample_t::sample_iterator i = sample.p_begin();
	 i != sample.p_end(); i++)
      for (typename Element<A, T>::monoid_elt_t::iterator j = i->begin();
	   j != i->end(); j++)
	alpha.insert(*j);
    
    for (typename sample_t::sample_iterator i = sample.n_begin();
	 i != sample.n_end(); i++)
      for (typename Element<A, T>::monoid_elt_t::iterator j = i->begin();
	   j != i->end(); j++)
	alpha.insert(*j);
    
    for (typename set<typename Element<A, T>::monoid_t::alphabet_t::letter_t>::iterator 
	   i = alpha.begin(); i != alpha.end(); i++)
      alphabet.insert(*i);
  }
  
  template <typename A, typename T, typename Kind_, typename Container_>
  void 
  give_alphabet(const Element<A, T>& a, 
		const Presentation<Kind_, Container_,
		typename Element<A, T>::monoid_elt_t>& sample,
		typename Element<A, T>::monoid_t::alphabet_t& alphabet)
  {
    typedef 
      Presentation<Kind_, Container_, typename Element<A, T>::monoid_elt_t>
      sample_t;

    set<typename Element<A, T>::monoid_t::alphabet_t::letter_t> alpha;

    for (typename sample_t::sample_iterator i = sample.p_begin();
	 i != sample.p_end(); i++)
      for (typename Element<A, T>::monoid_elt_t::const_iterator j = i->begin();
	   j != i->end(); j++)
	alpha.insert(*j);

    for (typename sample_t::sample_iterator i = sample.n_begin();
	 i != sample.n_end(); i++)
      for (typename Element<A, T>::monoid_elt_t::const_iterator j = i->begin();
	   j != i->end(); j++)
	alpha.insert(*j);

    for (typename set<typename Element<A, T>::monoid_t::alphabet_t::letter_t>::iterator 
	   i = alpha.begin(); i != alpha.end(); i++)
      alphabet.insert(*i);	 
  }
  
  
} // mical 

#endif // MICAL_GIVE_ALPHABET_HXX_
