// load_sample.hxx
//
// Mical, a regular grammar inference library
// Copyright (C) 2003 Maxime Rey
//
// This library is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public
// License as published by the Free Software Foundation; either
// version 2.1 of the License, or any later version.
//
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// Lesser General Public License for more details.

// You should have received a copy of the GNU Lesser General Public
// License along with this library; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

#ifndef MICAL_LOAD_SAMPLE_HXX_
# define MICAL_LOAD_SAMPLE_HXX_

#include <mical/samples/tools/load_sample.hh>
#include <vaucanson/algebra/concrete/predefs.hh>

namespace mical {  

  template <typename Kind_, typename Container_, typename Word_>
  void 
  load_presentation(Presentation<Kind_, Container_, Word_>& pres, 
		    const char* name_file)
  {
    
    std::ifstream is;
    char c;
    bool pos = false;
    
    Word_ str;

    is.open(name_file);

    while (is.good())     
      {
	c = is.get();     
      	switch (c)
	  {
	  case ' ' : 
	    {
	      break;
	    }
	  case '\n' :
	    {
	      if (pos)
		pres.add_positive(str);
	      else
		pres.add_negative(str);
	      str = "";
	      break;
	    }
	  case '+' :
	    {
	      pos = true;
	      break;
	    }
	  case '-' :
	    {
	      pos = false;
	      break;
	    }
	  default : 
	    {
	      str += c;
	      break;
	    }
	  }
      }

    is.close();           
  }
  
  template <typename Container_, typename Word_>
  void load_sample(FixedSample<Container_, Word_>& sample, 
		   const char* name_file, bool kind)
  {
    std::ifstream is;
    char c;
    bool pos = false;
    
    Word_ str;

    is.open(name_file);

    while (is.good())     
      {
	c = is.get();     
      	switch (c)
	  {
	  case ' ' : 
	    {
	      break;
	    }
	  case '\n' :
	    {
	      if (pos)
		sample.add_word(str);
	      str = "";
	      break;
	    }
	  case '+' :
	    {
	      pos = kind;
	      break;
	    }
	  case '-' :
	    {
	      pos = !kind;
	      break;
	    }
	  default : 
	    {
	      str += c;
	      break;
	    }
	  }
      }

    is.close();  
  }
 
  template <typename Container_, typename Word_>
  void 
  load_pos_sample(FixedSample<Container_, Word_>& sample, 
		       const char* name_file)
  {
    load_sample(sample, name_file, true);
  }

  template <typename Container_, typename Word_>
  void 
  load_neg_sample(FixedSample<Container_, Word_>& sample, 
		       const char* name_file)
  {
    load_sample(sample, name_file, false);
  }

} // mical 

namespace vcsn {

  void op_in_add(const algebra::FreeMonoid<algebra::char_letter::Alphabet>&, 
		 std::basic_string<char>& str, 
		 std::basic_string<char> elt)
  {
    str += elt;
  }

} // vcsn

#endif // MICAL_LOAD_SAMPLE_HXX_
