// order_search.hxx
//
// Mical, a regular grammar inference library
// Copyright (C) 2003 Maxime Rey
//
// This library is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public
// License as published by the Free Software Foundation; either
// version 2.1 of the License, or any later version.
//
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// Lesser General Public License for more details.

// You should have received a copy of the GNU Lesser General Public
// License along with this library; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

#ifndef MICAL_ORDER_SEARCH_HXX_
# define MICAL_ORDER_SEARCH_HXX_

#include <mical/walkers/order_search.hh>

namespace mical {

  template <class A, class T, class Task_, class Order_>
  void
  order_search(const Element<A, T>& a, const Order_& order, Task_& task)
  {
    Element<A, T> work = a;
    work.emancipate();
    
    order_search(work, order, task);
  }
  //////////////////////////////////////////////////////////////////////////////

  //////////////////////////////////////////////////////////////////////////////
  template <class A, class T, class Task_, class Order_>
  void
  order_search(Element<A, T>& a, const Order_& order, Task_& task)
  {
    order_search(a, *a.initial().begin(), order, task);
  }
  //////////////////////////////////////////////////////////////////////////////

  //////////////////////////////////////////////////////////////////////////////
  template <class A, class T, class Task_, class Order_>
  void
  order_search(const Element<A, T>& a, 
	       hstate_t init, 
	       const Order_& order, 
	       Task_& task)
  {
    Element<A, T> work = a;
    work.emancipate();
    
    order_search(work, init, order, task);
  }
  //////////////////////////////////////////////////////////////////////////////

  //////////////////////////////////////////////////////////////////////////////
  template <class A, class T, class Task_, class Order_>
  void
  order_search(Element<A, T>& a, 
	       hstate_t init, 
	       const Order_& order, 
	       Task_& task)
  {
    typedef InfoOrder<A, T, Task_, Order_> info_t;

    info_t info;

    info.auto_ = a;
    info.auto_.emancipate();

    info.state_ = init;
    info.task_ = task;
    info.order_ = order;
    
    order_search_(info);
    
    a = info.auto_;
    task = info.task_;  
  }
  //////////////////////////////////////////////////////////////////////////////

  //////////////////////////////////////////////////////////////////////////////
  template <class A, class T, class Task_, class Order_>
  void
  order_search_(InfoOrder<A, T, Task_, Order_>& info)
  {
    if (info.stop_)
      return ;

    hstate_t svg = info.state_;
    
    info.task_.prefix(info);
    info.state_ = info.order_.next(info);

    if (info.first_)
      info.first_ = false;

    order_search_(info);

    info.state_ = svg;
    info.task_.postfix(info);
  }
  //////////////////////////////////////////////////////////////////////////////

} // mical

#endif // MICAL_ORDER_SEARCH_HXX_
