// test_mca_to_pta.cc

#include <vaucanson/fundamental/predefs.hh>
#include <vaucanson/algebra/concrete/free_monoid/predefs.hh>
#include <vaucanson/algebra/concrete/semiring/numerical_semiring.hh>
#include <vaucanson/algebra/concrete/series/polynoms.hh>
#include <vaucanson/tools/usual.hh>

#include <mical/internal/make_mca.hh>
#include <mical/internal/mca_to_pta.hh>

#include <mical/samples/tools/load_sample.hh>

int main(int argc, char** argv)
{
  using namespace std;
  using namespace mical;
  using namespace vcsn;
  using namespace vcsn::algebra;
  using namespace vcsn::algebra::char_letter;

  Alphabet a;

  a.insert('a');
  a.insert('b');
  
  Words free_monoid(a);
  NumericalSemiring ring;
  Series<NumericalSemiring, Words> s(ring, free_monoid);

  typedef Automata auto_t;
  typedef 
    utility::ref< 
    AutomatonImpl<labels_are_series,
    Series<NumericalSemiring, Words>,
    polynom<Word::value_t, bool>,
    ManyLinks<polynom<Word::value_t, bool>, NoTag, NoTag>,
    NoTag,
    map> >
    implem_t;
  
  Element<auto_t, implem_t> A, B;

  A.create();
  B.create();

  A.series() = s;
  B.series() = s;

  Presentation<inference_kind::fixed, set<Word>, Word> pres;

  load_presentation(pres, "toy.a");

  make_mca(A, pres);

  mca_to_pta(A);

  SAVE_AUTOMATON_DOT_SIMPLE("pta1", A);


  FixedSample<set<Word>, Word> sample_pos;

  load_pos_sample(sample_pos, "toy.a");

  make_mca(B, sample_pos);

  mca_to_pta(B);

  SAVE_AUTOMATON_DOT_SIMPLE("pta2", B);

  return 0;
}
