// test_renumerotation.cc

#include <vaucanson/fundamental/predefs.hh>
#include <vaucanson/algebra/concrete/free_monoid/predefs.hh>
#include <vaucanson/algebra/concrete/semiring/numerical_semiring.hh>
#include <vaucanson/algebra/concrete/series/polynoms.hh>
#include <vaucanson/tools/usual.hh>

#include <mical/internal/make_mca.hh>
#include <mical/internal/make_pta.hh>
#include <mical/internal/renumerotation.hh>

#include <mical/samples/tools/load_sample.hh>

#include <utility>

int main(int argc, char** argv)
{
  using namespace std;
  using namespace mical;
  using namespace vcsn;
  using namespace vcsn::algebra;
  using namespace vcsn::algebra::char_letter;

  Alphabet a;

  a.insert('a');
  a.insert('b');
  
  Words free_monoid(a);
  NumericalSemiring ring;
  Series<NumericalSemiring, Words> s(ring, free_monoid);

  typedef Automata auto_t;
  typedef 
    utility::ref< 
    AutomatonImpl<labels_are_series,
    Series<NumericalSemiring, Words>,
    polynom<Word::value_t, bool>,
    ManyLinks<polynom<Word::value_t, bool>, NoTag, NoTag>,
    NoTag,
    map> >
    implem_t;
  
  Element<auto_t, implem_t> A, B, C;

  A.create();
  B.create();
  C.create();

  A.series() = s;
  B.series() = s;
  C.series() = s;

  map<hstate_t, unsigned> mappy;

  FixedSample<set<Word>, Word> sample_pos;

  load_pos_sample(sample_pos, "../framework/toy.a");

  make_mca(A, sample_pos);
  make_pta(B, sample_pos);
  make_pta(C, sample_pos, mappy);

  SAVE_AUTOMATON_DOT_SIMPLE("mca1", A);
  SAVE_AUTOMATON_DOT_SIMPLE("pta1", B);

  standard_renum(A);
  standard_renum(B);
    

  SAVE_AUTOMATON_DOT_SIMPLE("mca2", A);
  SAVE_AUTOMATON_DOT_SIMPLE("pta2", B);


  frequency_renum(C, mappy);

  SAVE_AUTOMATON_DOT_SIMPLE("pta_frequency", C);

  return 0;
}
