// test_quotient.cc

#include <vaucanson/fundamental/predefs.hh>
#include <vaucanson/algebra/concrete/free_monoid/predefs.hh>
#include <vaucanson/algebra/concrete/semiring/numerical_semiring.hh>
#include <vaucanson/algebra/concrete/series/polynoms.hh>
#include <vaucanson/tools/usual.hh>

#include <mical/primitives/quotient.hh>
#include <mical/primitives/compute_one.hh>


int main(int argc, char** argv)
{
  using namespace std;
  using namespace mical;
  using namespace vcsn;
  using namespace vcsn::algebra;
  using namespace vcsn::algebra::char_letter;

  Alphabet a;

  a.insert('a');
  a.insert('b');
  
  Words free_monoid(a);
  NumericalSemiring ring;
  Series<NumericalSemiring, Words> s(ring, free_monoid);

  typedef Automata auto_t;
  typedef 
    utility::ref< 
    AutomatonImpl<labels_are_series,
    Series<NumericalSemiring, Words>,
    polynom<Word::value_t, bool>,
    ManyLinks<polynom<Word::value_t, bool>, NoTag, NoTag>,
    NoTag,
    map> >
    implem_t;
  
  Element<auto_t, implem_t> A;

  A.create();

  hstate_t n_0 = A.add_state();
  hstate_t n_1 = A.add_state();

  A.set_initial(n_0);

  A.add_letter_edge(n_0, n_0, 'a');
  A.add_letter_edge(n_0, n_1, 'b');
  A.add_letter_edge(n_1, n_0, 'b');
  A.add_letter_edge(n_1, n_1, 'a');

  A.set_final(n_1);

  //  SAVE_AUTOMATON_DOT_SIMPLE("automaton", A);

  cout << "Words for state 0" << endl << endl;

  cout << quotient(A, "") << endl;
  cout << quotient(A, "a") << endl;
  cout << quotient(A, "abb") << endl;
  cout << quotient(A, "abab") << endl;
  cout << quotient(A, "abaab") << endl;
  cout << quotient(A, "abaaab") << endl;
  cout << quotient(A, "abaaababb") << endl;
  cout << quotient(A, "abaaabaabb") << endl;
  cout << quotient(A, "aabb") << endl;
  cout << quotient(A, "aabab") << endl;
  cout << quotient(A, "aabaab") << endl;
  cout << quotient(A, "aabaaab") << endl;
  cout << quotient(A, "aabaaababaaab") << endl;
  cout << quotient(A, "aabaaabaabab") << endl;
  cout << quotient(A, "bb") << endl;
  cout << quotient(A, "bab") << endl;
  cout << quotient(A, "baab") << endl;
  cout << quotient(A, "baaab") << endl;
  cout << quotient(A, "baaababaaaaab") << endl;
  cout << quotient(A, "baaabaabb") << endl;
  cout << quotient(A, "bba") << endl;
  cout << quotient(A, "babaa") << endl;
  cout << quotient(A, "baabaaa") << endl;
  cout << quotient(A, "baaaba") << endl;
  cout << quotient(A, "baaababab") << endl;
  cout << quotient(A, "baaabaabaabaaa") << endl;
  cout << quotient(A, "aababaa") << endl;
  cout << quotient(A, "aabaabaaaaaa") << endl;

  cout << endl;
  cout << "Words for state 1" << endl << endl;

  cout << quotient(A, "b") << endl;
  cout << quotient(A, "ab") << endl;
  cout << quotient(A, "aba") << endl;
  cout << quotient(A, "abaa") << endl;
  cout << quotient(A, "abaaa") << endl;
  cout << quotient(A, "abaaabab") << endl;
  cout << quotient(A, "abaaabaab") << endl;
  cout << quotient(A, "aab") << endl;
  cout << quotient(A, "aaba") << endl;
  cout << quotient(A, "aabaa") << endl;
  cout << quotient(A, "aabaaa") << endl;
  cout << quotient(A, "aabaaabab") << endl;
  cout << quotient(A, "aabaaabaabbb") << endl;
  cout << quotient(A, "ba") << endl;
  cout << quotient(A, "baa") << endl;
  cout << quotient(A, "baaa") << endl;
  cout << quotient(A, "baaababbbaababaa") << endl;
  cout << quotient(A, "baaabaab") << endl;


  return 0;
}
