// test_get_kernel.cc

#include <mical/samples/services/get_kernel.hh>

#include <vaucanson/fundamental/predefs.hh>
#include <vaucanson/algebra/concrete/free_monoid/predefs.hh>
#include <vaucanson/algebra/concrete/semiring/numerical_semiring.hh>
#include <vaucanson/algebra/concrete/series/polynoms.hh>
#include <vaucanson/tools/usual.hh>

#include <mical/samples/tools/load_sample.hh>
#include <mical/internal/make_pta.hh>

#include <set>
#include <vector>
#include <string>
#include <iostream>

int main(int argc, char** argv)
{
  using namespace std;
  using namespace mical;
  using namespace vcsn;
  using namespace vcsn::algebra;
  using namespace vcsn::algebra::char_letter;

  Alphabet a;

  a.insert('a');
  a.insert('b');
  
  Words free_monoid(a);
  NumericalSemiring ring;
  Series<NumericalSemiring, Words> s(ring, free_monoid);

  typedef Automata auto_t;
  typedef 
    utility::ref< 
    AutomatonImpl<labels_are_series,
    Series<NumericalSemiring, Words>,
    polynom<Word::value_t, bool>,
    ManyLinks<polynom<Word::value_t, bool>, NoTag, NoTag>,
    NoTag,
    map> >
    implem_t;
  
  Element<auto_t, implem_t> A, B;

  A.create();

  hstate_t n_0 = A.add_state();
  hstate_t n_1 = A.add_state();

  A.set_initial(n_0);

  A.add_letter_edge(n_0, n_0, 'a');
  A.add_letter_edge(n_0, n_1, 'b');
  A.add_letter_edge(n_1, n_0, 'b');
  A.add_letter_edge(n_1, n_1, 'a');

  A.set_final(n_1);

  cout << "TESTS on a first automaton" << endl;

  FixedSample<set<Word>, Word> sample_pos;
  set<Word>                    container;

  get_kernel(A, sample_pos.container());
  get_kernel(A, container);
  
  cout << "With FixedSample" << endl;
  copy(sample_pos.begin(), sample_pos.end(), ostream_iterator<Word>(cout, "\n"));
  cout << "With a set" << endl;
  copy(container.begin(), container.end(), ostream_iterator<Word>(cout, "\n"));

  cout << endl;

  cout << "TESTS on a second automaton" << endl;

  B.create();

  sample_pos.clear();
  load_pos_sample(sample_pos, "sample2.toy");

  make_pta(B, sample_pos);

  container.clear();

  get_kernel(B, container);

  copy(container.begin(), container.end(), ostream_iterator<Word>(cout, "\n"));

  return 0;
}
